# -*- coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta
from __future__ import print_function
from __future__ import unicode_literals
from __future__ import absolute_import
from __future__ import division

from iobase import xmldata
from ionas import request
from fal_info import (FAL_SERVICECTRL_PATH, Results, ReqTypes)


# def append_service(service_name, categories=[]):
def append_service(service_name):

    PACKAGE_SERVICE_XML = '<service enable="false"><name>%s</name></service>' % (service_name,)
    # PACKAGE_SERVICE_IN_CATEGORY_XML = '<service><name>%s</name></service>' % (service_name,)

    servicectrl_config = __send_request(FAL_SERVICECTRL_PATH, ReqTypes.LOAD).get()

    if __is_service_registered(servicectrl_config, service_name):
        return True

    service = xmldata.XmlConfNode(PACKAGE_SERVICE_XML)
    servicectrl_config.get_node('servicelist').append_child(service)

    # service_in_category = xmldata.XmlConfNode(PACKAGE_SERVICE_IN_CATEGORY_XML)
    # for _category in categories:
    #     _services_node = servicectrl_config.get_node('categorylist/category/name[text()="{category}"]/../servicelist'.format(category=_category))
    #     _services_node.append_child(service_in_category.get_clone())

    result = __send_request(
        FAL_SERVICECTRL_PATH,
        ReqTypes.SAVE,
        servicectrl_config,
    ).get_result()
    if result != Results.OK:
        return False
    return True


def remove_service(service_name):
    try:
        entry = __send_request(FAL_SERVICECTRL_PATH, ReqTypes.LOAD)
        servicecontrol_node = entry.get()
        services_node = servicecontrol_node.get_node('servicelist')
        for (index, service_node) in enumerate(services_node.get_node_list('service')):
            try:
                if service_node.get_node('name').get_text() == service_name:
                    services_node.delete_child('service', index=index)
                    __send_request(
                        FAL_SERVICECTRL_PATH,
                        ReqTypes.SAVE,
                        servicecontrol_node,
                    )
                    return
            except:
                pass
    except:
        pass


def start_service(service_name, raise_exception=False):
    try:
        entry = request.Entry(ReqTypes.START, '<servicectrl><servicelist><service><name/></service></servicelist></servicectrl>')
        servicecontrol_node = entry.get()
        services_node = servicecontrol_node.get_node('servicelist')
        service_node = services_node.get_node('service')
        name_node = service_node.get_node('name')
        name_node.set_text(service_name)
        _entry = request.Request(FAL_SERVICECTRL_PATH).request(entry).get_entry()
        _result = _entry.get_result()
        _success = _result in (Results.OK,)
        if not _success:
            raise Exception('FALError')
        return _success
    except Exception as e:
        if raise_exception:
            raise
        return False


def stop_service(service_name, raise_exception=False):
    try:
        entry = request.Entry(ReqTypes.STOP, '<servicectrl><servicelist><service><name/></service></servicelist></servicectrl>')
        servicecontrol_node = entry.get()
        services_node = servicecontrol_node.get_node('servicelist')
        service_node = services_node.get_node('service')
        name_node = service_node.get_node('name')
        name_node.set_text(service_name)
        _entry = request.Request(FAL_SERVICECTRL_PATH).request(entry).get_entry()
        _result = _entry.get_result()
        _success = _result in (Results.OK,)
        if not _success:
            raise Exception('FALError')
        return _success
    except Exception as e:
        if raise_exception:
            raise
        return False


def __is_service_registered(servicectrl_config, service_name):
    PACKAGE_SERVICE_NAME_XPATH = 'servicelist/service/name[text()="%s"]' % (service_name,)
    try:
        servicectrl_config.get_node(PACKAGE_SERVICE_NAME_XPATH)
        return True
    except:
        return False


def __send_request(target, type, body=""):
    entry = request.Entry(type, body)
    return request.Request(target).request(entry).get_entry()

