"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Environment = void 0;
process.env["NODE_CONFIG_DIR"] = `${__dirname}/../config`;
const config = require("config");
const path = require("path");
class Environment {
    constructor() {
        if (!process.env.LAMBDA_TASK_ROOT) {
            this.userDir = process.env[process.platform == "win32" ? "USERPROFILE" : "HOME"];
        }
    }
    /**
     * セットアップ
     */
    setup() {
        this.optimazeConfig();
    }
    /**
     * Configの最適化
     */
    optimazeConfig() {
        if (process.env.LAMBDA_TASK_ROOT) {
            this.setConfigFromEnv('config.port.connectorForward', 'PORT_CONNECTOR_FORWARD');
            this.setConfigFromEnv('config.browser.sandbox', 'BROWSER_SANDBOX', 'INT');
            this.setConfigFromEnv('config.browser.screenshot.captureSamplingInterval', 'BROWSER_SCREENSHOT_CAPTURE_SAMPLING_INTERVAL', 'INT');
            this.setConfigFromEnv('config.browser.storageQuota', 'BROWSER_STORAGE_QUOTA', 'INT');
            this.setConfigFromEnv('config.webSocket.transports_ws_only', 'WEBSOCKET_TRANSPORTS_WS_ONLY', 'INT');
            this.setConfigFromEnv('config.storage.path', 'STORAGE_PATH');
            this.setConfigFromEnv('config.fileRelayServer.endpoint', 'FILE_RELAY_SERVER_ENDPOINT');
            this.setConfigFromEnv('config.fileRelayServer.port', 'FILE_RELAY_SERVER_PORT', 'INT');
            this.setConfigFromEnv('config.apiServer.endpoint', 'API_SERVER_ENDPOINT');
            this.setConfigFromEnv('config.apiServer.port', 'API_SERVER_PORT', 'INT');
        }
        else {
            config.apiRelayClient.agent.fileName = this.convertRelativePathToUserPath(config.apiRelayClient.agent.fileName);
            config.apiRelayClient.status.fileName = this.convertRelativePathToUserPath(config.apiRelayClient.status.fileName);
            config.credentials.fileName = this.convertRelativePathToUserPath(config.credentials.fileName);
            config.hardware.fileName = this.convertRelativePathToUserPath(config.hardware.fileName);
            config.lang.fileName = this.convertRelativePathToUserPath(config.lang.fileName);
            config.profile.fileName = this.convertRelativePathToUserPath(config.profile.fileName);
            config.proxy.fileName = this.convertRelativePathToUserPath(config.proxy.fileName);
            config.events.folderName = this.convertRelativePathToUserPath(config.events.folderName);
            config.storage.path = this.convertRelativePathToUserPath(config.storage.path);
        }
    }
    /**
     * 環境変数からConfigを設定
     */
    setConfigFromEnv(configKey, envKey, cast) {
        let value = process.env[envKey];
        if (value === undefined) {
            return;
        }
        if (cast == 'INT') {
            value = parseInt(value);
        }
        const keys = configKey.split('.');
        keys.shift();
        let target = config;
        for (let i = 0; i < keys.length - 1; i++) {
            target = target[keys[i]];
        }
        if (process.env[envKey] !== '') {
            target[keys[keys.length - 1]] = value;
        }
        else {
            delete target[keys[keys.length - 1]];
        }
    }
    ;
    /**
     * 相対パスをユーザパスに変更
     */
    convertRelativePathToUserPath(_path) {
        if (_path && _path.indexOf('/') !== 0) {
            return path.join(this.userDir, `.${config.productId}`, _path);
        }
        return _path;
    }
}
exports.Environment = Environment;
