"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.FileRelayConnector = void 0;
const config = require("config");
const express = require("express");
const http = require("http");
const redis = require("redis");
const httpClient_1 = require("../httpClient");
const RelayTokenStore_1 = require("../socket-io.apiRelay/RelayTokenStore");
const SocketIOTunnel = require("socket.io-tunnel-stream");
const socket_io_apiRelay_1 = require("../socket-io.apiRelay");
class FileRelayConnector {
    constructor(mode, localPort) {
        this.express = null;
        this.mode = mode;
        this.localPort = localPort;
        if (mode == 'instant') {
            this.bootInstantHttpServer();
        }
    }
    connect(event) {
        return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
            const url = `${config.fileRelayServer.endpoint}:${config.fileRelayServer.port}`;
            const httpAgent = yield httpClient_1.default.getHttpAgentForSocketIo(`${url}/socket.io/socket.io.js`);
            this.socketIoRelay = new socket_io_apiRelay_1.SocketIoRelay(url, httpAgent, {
                transportsWsOnly: config.webSocket.transports_ws_only,
                timeout: 15000,
                reconnectionAttempts: 4
            });
            try {
                // WS中継Listen要求
                yield this.socketIoRelay.listen('127.0.0.1', this.localPort, event.token.id);
            }
            catch (e) {
                console.log(`Failed WS relay Listen request.: ${JSON.stringify(e)}`);
                this.socketIoRelay.close();
            }
            this.socketIoRelay.on('connect', (socket) => {
                console.log('Connected to WS relay server.');
                this.tunnelBuilder = new SocketIOTunnel.TunnelBuilder(socket);
            });
            this.socketIoRelay.on('relay_disconnect', () => __awaiter(this, void 0, void 0, function* () {
                console.log('Disconnect from WS relay server.');
                if (this.mode == 'instant') {
                    this.server.close(() => {
                        resolve();
                    });
                }
            }));
            RelayTokenStore_1.default.put(event.token);
            // pubsub接続情報がある場合はpublish
            if (event.pubsubChannel) {
                const pub = redis.createClient(event.pubsubPort, event.pubsubHost);
                pub.publish(event.pubsubChannel, JSON.stringify({ result: 0, tokenId: event.token.id }), () => {
                    pub.quit();
                });
            }
            if (this.mode == 'instance') {
                resolve();
            }
        }));
    }
    bootInstantHttpServer() {
        return new Promise((resolve, reject) => {
            this.express = express();
            this.express.use('/storage', express.static(config.storage.path));
            this.server = new http.Server(this.express);
            this.server.listen(this.localPort, () => {
                console.log('Started instant http server.');
                resolve();
            });
        });
    }
}
exports.FileRelayConnector = FileRelayConnector;
