"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.FileUploader = void 0;
const config = require("config");
const fs = require("fs");
const redis = require("redis");
const cbot_api_client_1 = require("cbot-api-client");
const httpClient_1 = require("../httpClient");
const utils_1 = require("../utils");
;
class FileUploader {
    constructor(mode, event) {
        this.mode = mode;
        this.event = event;
    }
    preparate() {
        return __awaiter(this, void 0, void 0, function* () {
            let result = 0, message;
            const tasks = [];
            try {
                for (const ref in this.event.tokenKeys) {
                    if (ref.indexOf('.') !== -1) {
                        console.log('Invalid ref:', ref);
                        throw new Error('Invalid ref');
                    }
                    const parsedRef = ref.split('/');
                    const fileId = parsedRef[parsedRef.length - 1];
                    const tempPath = yield utils_1.makeStorageDirAndTryCallback(`${this.event.contractId}/uploads`, (fullPath) => __awaiter(this, void 0, void 0, function* () {
                        const tempPath = `${fullPath}/${fileId}`;
                        yield fs.promises.rename(`${config.storage.path}/${this.event.contractId}/${ref}`, tempPath);
                        return tempPath;
                    }));
                    const { key, timestamp } = this.event.tokenKeys[ref];
                    const filename = (yield fs.promises.readdir(`${tempPath}`))[0];
                    if (!fs.existsSync(`${tempPath}/${filename}`)) {
                        throw new Error('Upload file not found');
                    }
                    const params = {
                        key: key,
                        filename: filename,
                        timestamp: timestamp
                    };
                    tasks.push({
                        filePath: tempPath,
                        fileName: filename,
                        params: params
                    });
                }
                return tasks;
            }
            catch (e) {
                result = -1;
                message = e.message;
                console.log(e.message);
                throw e;
            }
            finally {
                // pubsub接続情報がある場合はpublish
                if (this.event.pubsubChannel) {
                    const body = {
                        result: result
                    };
                    if (result != 0) {
                        body.message = message;
                    }
                    const pub = redis.createClient(this.event.pubsubPort, this.event.pubsubHost);
                    pub.publish(this.event.pubsubChannel, JSON.stringify(body), () => {
                        pub.quit();
                    });
                }
            }
        });
    }
    upload(tasks) {
        return __awaiter(this, void 0, void 0, function* () {
            const url = `${config.apiServer.endpoint}:${config.apiServer.port}`;
            const httpAgent = yield httpClient_1.default.getHttpAgentForSocketIo(`${url}/socket.io/socket.io.js`);
            const client = new cbot_api_client_1.default({
                endpoint: url,
                lang: 'en',
                agent: httpAgent
            });
            yield client.ws.connect(this.event.tokenId);
            try {
                console.log('Start upload');
                for (const task of tasks) {
                    console.log('upload:', task);
                    if (!fs.existsSync(`${task.filePath}/${task.fileName}`)) {
                        console.log('Upload file not found', `${task.filePath}/${task.fileName}`);
                        continue;
                    }
                    const stream = fs.createReadStream(`${task.filePath}/${task.fileName}`);
                    yield client.ws.upload(stream, task.params);
                    yield fs.promises.rmdir(task.filePath, {
                        recursive: true
                    });
                }
                console.log('Uploaded');
            }
            catch (e) {
                console.log(e.message);
                throw e;
            }
            finally {
                client.ws.disconnect();
            }
        });
    }
}
exports.FileUploader = FileUploader;
