"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const axios_1 = require("axios");
const crypto = require("crypto");
const http = require("http");
const https = require("https");
const url = require("url");
const http_proxy_agent_1 = require("http-proxy-agent");
const https_proxy_agent_1 = require("https-proxy-agent");
const digest_proxy_agent_1 = require("digest-proxy-agent");
const pac_proxy_agent_1 = require("pac-proxy-agent");
class Challenge {
    constructor(challenge, auth) {
        this.auth = auth;
        this.nc = 1;
        this.data = parseChallenge(challenge);
    }
    /**
     * ダイジェスト生成
     */
    createDigest(method, uri) {
        const md5 = (str) => {
            const md5 = crypto.createHash('md5');
            return md5.update(str, 'binary').digest('hex');
        };
        const cnonce = md5(String(new Date().getTime()));
        const parsed = url.parse(uri);
        const path = parsed.path;
        const nc = this.getNc();
        const a1 = `${this.auth.username}:${this.data.realm}:${this.auth.password}`;
        const a2 = `${method}:${path}`;
        const response = `${md5(a1)}:${this.data.nonce}:${nc}:${cnonce}:${this.data.qop}:${md5(a2)}`;
        const digest = `Digest username="${this.auth.username}", realm="${this.data.realm}", nonce="${this.data.nonce}", uri="${path}", algorithm=MD5, response="${md5(response)}", qop=${this.data.qop}, nc=${nc}, cnonce="${cnonce}"`;
        return digest;
    }
    /**
     * NC取得
     */
    getNc() {
        return `${++this.nc}`.padStart(8, '0');
    }
}
function parseChallenge(challenge) {
    const data = {};
    const matches = challenge.match(/^digest (.+)$/i);
    if (matches) {
        matches[1].split(',').forEach((item) => {
            item = item.trim();
            let key, value;
            [key, value] = item.split('=');
            data[key] = JSON.parse(value);
        });
    }
    return data;
}
class HttpClient {
    constructor() {
        this.getProxy = () => __awaiter(this, void 0, void 0, function* () {
            return null;
        });
        this.requestTimeout = 10000;
        axios_1.default.defaults.timeout = this.requestTimeout;
        this.resetAuth();
    }
    /**
     * 初期化
     */
    init(getProxy) {
        if (getProxy) {
            this.getProxy = getProxy;
        }
    }
    /**
     * Methodエイリアス
     */
    get(url, config = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.request('get', url, null, config);
        });
    }
    post(url, data, config = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.request('post', url, data, config);
        });
    }
    put(url, data, config = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.request('put', url, data, config);
        });
    }
    delete(url, data, config = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.request('delete', url, data, config);
        });
    }
    /**
     * リクエスト処理
     */
    request(method, url, data, config = {}) {
        var _a, _b, _c, _d;
        return __awaiter(this, void 0, void 0, function* () {
            const httpAgent = yield this.getHttpAgent(url);
            if (this.proxyChallenge) {
                if (httpAgent instanceof http_proxy_agent_1.HttpProxyAgent || httpAgent instanceof https_proxy_agent_1.HttpsProxyAgent) {
                    // プロキシ設定が変わっていたらDigestProxyAgentを破棄
                    const proxy = yield this.getProxy();
                    if (proxy.url != this.proxy) {
                        this.digestProxyAgent = null;
                        this.proxy = proxy.url;
                    }
                    // DigestProxyAgent生成
                    if (!this.digestProxyAgent) {
                        this.digestProxyAgent = new digest_proxy_agent_1.DigestProxyAgent(this.proxy, this.proxyChallenge);
                    }
                }
                else {
                    this.resetAuth();
                }
            }
            try {
                const _httpAgent = (_a = this.digestProxyAgent) !== null && _a !== void 0 ? _a : httpAgent;
                Object.assign(config, {
                    method: method,
                    url: url,
                    data: data,
                    httpAgent: _httpAgent,
                    httpsAgent: _httpAgent
                });
                const response = yield this.cancelTokenScope((cancelToken) => __awaiter(this, void 0, void 0, function* () {
                    return yield axios_1.default(Object.assign({}, config, { cancelToken: cancelToken }));
                }), this.requestTimeout);
                console.log(`http-agent: ${_httpAgent === null || _httpAgent === void 0 ? void 0 : _httpAgent.constructor.name}`);
                return response;
            }
            catch (e) {
                try {
                    // 認証エラー
                    if (((_b = e.response) === null || _b === void 0 ? void 0 : _b.status) == 401) {
                        const headers = e.response.headers;
                        const challenge = headers['www-authenticate'];
                        if (!challenge) {
                            throw e;
                        }
                        if (this.authType == 'basic') {
                            // Basic認証からDigest認証に切り替え
                            if (challenge.match(/^digest /i)) {
                                this.authType = 'digest';
                                if (!config.auth) {
                                    throw e;
                                }
                                const challengeInstance = new Challenge(challenge, config.auth);
                                const digest = challengeInstance.createDigest(method, url);
                                config.headers['Authorization'] = digest;
                                config['_auth'] = config.auth;
                                delete config.auth;
                            }
                            // Basic認証失敗
                            else {
                                throw e;
                            }
                        }
                        else {
                            // Digest認証失敗
                            if (method == 'get' || method == 'post' || method == 'put' || method == 'delete') {
                                // メソッド名を大文字にしてリトライ(サーバによって小文字大文字どちらで処理するか違うため)
                                method = method.toUpperCase();
                                config.auth = config['_auth'];
                                delete config['_auth'];
                                this.authType = 'basic';
                            }
                            else {
                                throw e;
                            }
                        }
                        return yield this.request(method, url, data, config);
                    }
                    // プロキシ認証エラー
                    else if (((_c = e.response) === null || _c === void 0 ? void 0 : _c.status) == 407) {
                        const challenge = (_d = e.response) === null || _d === void 0 ? void 0 : _d.headers['proxy-authenticate'];
                        if (!challenge) {
                            throw e;
                        }
                        if (this.proxyAuthType == 'basic') {
                            // Basic認証からDigest認証に切り替え
                            if (challenge.match(/^digest /i)) {
                                this.proxyAuthType = 'digest';
                                this.proxyChallenge = challenge;
                            }
                            // Basic認証失敗
                            else {
                                throw e;
                            }
                        }
                        else {
                            // Digest認証からBasic認証に切り替え
                            if (challenge.match(/^basic /i)) {
                                this.resetAuth();
                            }
                            // Digest認証失敗
                            else {
                                throw e;
                            }
                        }
                        return yield this.request(method, url, data, config);
                    }
                    else {
                        throw e;
                    }
                }
                catch (e) {
                    this.resetAuth();
                    throw e;
                }
            }
        });
    }
    /**
     * 認証情報をリセット
     */
    resetAuth() {
        this.authType = 'basic';
        this.proxyAuthType = 'basic';
        this.proxyChallenge = null;
        this.digestProxyAgent = null;
    }
    /**
     * socket.io用のhttpAgentを取得
     */
    getHttpAgentForSocketIo(requestUrl) {
        var _a, _b;
        return __awaiter(this, void 0, void 0, function* () {
            const httpAgent = yield this.getHttpAgent(requestUrl);
            let _httpAgent;
            try {
                // Http(s)ProxyAgent以外の場合は採用
                if (!(httpAgent instanceof http_proxy_agent_1.HttpProxyAgent) && !(httpAgent instanceof https_proxy_agent_1.HttpsProxyAgent)) {
                    _httpAgent = httpAgent;
                    return _httpAgent;
                }
                try {
                    // Http(s)ProxyAgentで通信できれば採用
                    yield this.cancelTokenScope((cancelToken) => __awaiter(this, void 0, void 0, function* () {
                        yield axios_1.default.get(requestUrl, {
                            httpAgent: httpAgent,
                            httpsAgent: httpAgent,
                            cancelToken: cancelToken
                        });
                    }), this.requestTimeout);
                    _httpAgent = httpAgent;
                    return _httpAgent;
                }
                catch (e) {
                    if (((_a = e.response) === null || _a === void 0 ? void 0 : _a.status) != 407) {
                        throw e;
                    }
                    // チャレンジがあればDigestProxyAgentを採用
                    const challenge = (_b = e.response) === null || _b === void 0 ? void 0 : _b.headers['proxy-authenticate'];
                    if (!challenge) {
                        throw e;
                    }
                    const proxy = yield this.getProxy();
                    _httpAgent = new digest_proxy_agent_1.DigestProxyAgent(proxy.url, challenge);
                    return _httpAgent;
                }
            }
            catch (e) {
                throw e;
            }
            finally {
                if (_httpAgent) {
                    console.log(`http-agent: ${_httpAgent === null || _httpAgent === void 0 ? void 0 : _httpAgent.constructor.name}`);
                }
            }
        });
    }
    /**
     * キャンセルトークンの生成とハンドリングを行うスコープ
     */
    cancelTokenScope(callback, timeout) {
        return __awaiter(this, void 0, void 0, function* () {
            const source = axios_1.default.CancelToken.source();
            const cancel = setTimeout(() => {
                source.cancel();
            }, timeout);
            try {
                return yield callback(source.token);
            }
            catch (e) {
                throw e;
            }
            finally {
                clearTimeout(cancel);
            }
        });
    }
    /**
     * HTTP Agent取得
     */
    getHttpAgent(requestUrl) {
        return __awaiter(this, void 0, void 0, function* () {
            const proxy = yield this.getProxy();
            const destination = url.parse(requestUrl);
            const protocol = destination.protocol === 'ws:' ? 'http:' : (destination.protocol === 'wss:' ? 'https:' : destination.protocol);
            if (proxy === null || proxy === void 0 ? void 0 : proxy.enable) {
                if (!proxy.pac) {
                    const urlProxy = url.parse(proxy.url);
                    if (protocol == 'https:') {
                        return new https_proxy_agent_1.HttpsProxyAgent(urlProxy);
                    }
                    return new http_proxy_agent_1.HttpProxyAgent(urlProxy);
                }
                else {
                    return new pac_proxy_agent_1.PacProxyAgent(proxy.pac);
                }
            }
            else {
                if (protocol == 'https:') {
                    return new https.Agent({
                        rejectUnauthorized: false
                    });
                }
                return new http.Agent();
            }
        });
    }
    /**
     * 新規インスタンスを取得
     */
    getNewInstance() {
        return new HttpClient;
    }
}
exports.default = new HttpClient;
