"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.handler = void 0;
const environment_1 = require("./environment");
const connector_1 = require("./fileRelay/connector");
const uploader_1 = require("./fileUploader/uploader");
(new environment_1.Environment).setup();
/**
 * Lambda実行のイベントハンドラ
 */
const handler = (event, context) => __awaiter(void 0, void 0, void 0, function* () {
    try {
        console.log(event);
        // 仮想ブラウザ起動
        if (event.behavior == 'browser') {
            yield bootVirtualBrowser(event);
        }
        // ファイル中継起動
        else if (event.behavior == 'file-relay') {
            yield bootFileRelay(event);
        }
        // ファイルアップロード
        else if (event.behavior == 'file-upload') {
            yield fileUploader(event);
        }
        context.succeed({
            result: 0,
        });
    }
    catch (e) {
        context.fail(e.message);
    }
});
exports.handler = handler;
/**
 * 仮想ブラウザ起動(Lambda用)
 */
const bootVirtualBrowser = (event) => __awaiter(void 0, void 0, void 0, function* () {
    console.log('start browser process.');
    const { Browser } = require('./virtualThings/browser');
    const browser = new Browser('instant');
    yield browser.start(event);
    console.log('end browser process.');
});
/**
 * ファイル中継起動(Lambda用)
 */
const bootFileRelay = (event) => __awaiter(void 0, void 0, void 0, function* () {
    console.log('start file-relay process.');
    const fileRelayConnector = new connector_1.FileRelayConnector('instant', 15080);
    yield fileRelayConnector.connect(event);
    console.log('end file-relay process.');
});
/**
 * ファイルアップロード(Lambda用)
 */
const fileUploader = (event) => __awaiter(void 0, void 0, void 0, function* () {
    console.log('start file-upload process.');
    const fileUploader = new uploader_1.FileUploader('instant', event);
    const tasks = yield fileUploader.preparate();
    yield fileUploader.upload(tasks);
    console.log('end file-upload process.');
});
/**
 * アプリ起動
 */
if (!process.env.LAMBDA_TASK_ROOT) {
    (() => __awaiter(void 0, void 0, void 0, function* () {
        const { App } = require('./app');
        const app = yield (new App).init();
        yield app.start();
    }))();
}
