"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.LocalStorageCleaner = void 0;
const fs = require("fs");
const moment = require("moment");
class LocalStorageCleaner {
    constructor(targets) {
        this.targets = targets;
    }
    exec() {
        return __awaiter(this, void 0, void 0, function* () {
            const currentMoment = moment();
            console.log('Start LocalStorageCleaner', currentMoment.toString());
            for (const target of this.targets) {
                try {
                    yield this.scanDir(target.path, target, currentMoment);
                }
                catch (e) {
                    console.log(e.message);
                }
            }
            ;
            console.log('End LocalStorageCleaner', moment().toString());
        });
    }
    scanDir(path, target, currentMoment) {
        return __awaiter(this, void 0, void 0, function* () {
            if (fs.existsSync(path)) {
                const stats = yield fs.promises.stat(path);
                const files = yield fs.promises.readdir(path);
                for (const file of files) {
                    const stats = yield fs.promises.stat(`${path}/${file}`);
                    if (stats.isDirectory()) {
                        yield this.scanDir(`${path}/${file}`, target, currentMoment);
                    }
                    else {
                        if (stats.mtimeMs + target.timeout < currentMoment.valueOf()) {
                            yield this.unlink(`${path}/${file}`);
                        }
                    }
                }
                const _files = yield fs.promises.readdir(path);
                if (!_files.length && path != target.path) {
                    if (stats.mtimeMs + target.timeout < currentMoment.valueOf()) {
                        yield this.rmdir(path);
                    }
                }
            }
        });
    }
    unlink(path) {
        return __awaiter(this, void 0, void 0, function* () {
            console.log('unlink', path);
            yield fs.promises.unlink(path);
        });
    }
    rmdir(path) {
        return __awaiter(this, void 0, void 0, function* () {
            console.log('rmdir', path);
            yield fs.promises.rmdir(`${path}`);
        });
    }
}
exports.LocalStorageCleaner = LocalStorageCleaner;
