"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.validateErrorResponse = exports.API = void 0;
const config = require("config");
const express = require("express");
const i18n_1 = require("i18n");
const nodeUuid = require("node-uuid");
const passport = require("passport");
const express_validator_1 = require("express-validator");
const browser_1 = require("../../virtualThings/browser");
const connector_1 = require("../../virtualThings/connector");
const connector_2 = require("../../fileRelay/connector");
const uploader_1 = require("../../fileUploader/uploader");
const httpClient_1 = require("../../httpClient");
const utils_1 = require("../../utils");
const cancelRegistrationRequest_1 = require("./requests/cancelRegistrationRequest");
const langRequest_1 = require("./requests/langRequest");
const passwordRequest_1 = require("./requests/passwordRequest");
const profileRequest_1 = require("./requests/profileRequest");
const proxyRequest_1 = require("./requests/proxyRequest");
const registrationRequest_1 = require("./requests/registrationRequest");
const pinRequest_1 = require("./requests/pinRequest");
const eventRequest_1 = require("./requests/eventRequest");
const releaseRegisterRequest_1 = require("./requests/releaseRegisterRequest");
class API {
    constructor(httpServer) {
        this.httpServer = httpServer;
        this.express = this.httpServer.express;
        this.router();
    }
    /**
     * 認可
     */
    isAuthenticated(req, res, next) {
        return __awaiter(this, void 0, void 0, function* () {
            if (req.header('x-agent-id') !== undefined) {
                const agent = yield this.httpServer.app.getAgent();
                if (req.header('x-agent-id') === agent.id && req.header('x-secret') === agent.secret) {
                    return next();
                }
            }
            else if (req.isAuthenticated()) {
                return next();
            }
            this.apiResponse(res, 401, { message: 'Unauthorized' });
        });
    }
    /**
     * Router
     */
    router() {
        return __awaiter(this, void 0, void 0, function* () {
            this.guestRoute();
            this.authedRoute();
        });
    }
    /**
     * ゲストルート
     */
    guestRoute() {
        const guest = express.Router();
        this.express.use('/api', guest);
        guest.get('/healthcheck', (req, res) => {
            this.healthCheck(req, res);
        });
        guest.post('/agent/lang/initialize', (new langRequest_1.default(this)).validate(), (req, res) => {
            this.initLang(req, res);
        });
        guest.post('/agent/password/initialize', (new passwordRequest_1.default(this)).validate(true), (req, res) => {
            this.initPassword(req, res);
        });
        guest.post('/agent/login', passport.authenticate('custom'), (req, res) => {
            this.apiResponse(res, 200);
        });
    }
    /**
     * 認証済みルート
     */
    authedRoute() {
        const authed = express.Router();
        this.express.use('/api', authed);
        authed.use((req, res, next) => __awaiter(this, void 0, void 0, function* () { yield this.isAuthenticated(req, res, next); }));
        authed.get('/agent/version', (req, res) => __awaiter(this, void 0, void 0, function* () {
            yield this.versionCheck(req, res);
        }));
        authed.get('/agent/info', (req, res) => __awaiter(this, void 0, void 0, function* () {
            yield this.info(req, res);
        }));
        authed.post('/agent/connect', (req, res) => __awaiter(this, void 0, void 0, function* () {
            yield this.connectAgent(req, res);
        }));
        authed.delete('/agent/connect', (req, res) => __awaiter(this, void 0, void 0, function* () {
            yield this.disconnectAgent(req, res);
        }));
        authed.post('/agent/reset', (req, res) => __awaiter(this, void 0, void 0, function* () {
            yield this.resetAgent(req, res);
        }));
        authed.post('/agent/initialize', (req, res) => __awaiter(this, void 0, void 0, function* () {
            yield this.initializeAgent(req, res);
        }));
        authed.post('/agent/registration_requests', (new registrationRequest_1.default(this)).validate(), (req, res) => __awaiter(this, void 0, void 0, function* () {
            yield this.registrationRequest(req, res);
        }));
        authed.delete('/agent/registration_requests', (new cancelRegistrationRequest_1.default(this)).validate(), (req, res) => __awaiter(this, void 0, void 0, function* () {
            yield this.cancelRegistrationRequest(req, res);
        }));
        authed.post('/agent/pin', (new pinRequest_1.default(this)).validate(), (req, res) => __awaiter(this, void 0, void 0, function* () {
            yield this.pinRequest(req, res);
        }));
        authed.delete('/agent/registers', (new releaseRegisterRequest_1.default(this)).validate(), (req, res) => __awaiter(this, void 0, void 0, function* () {
            yield this.releaseRegister(req, res);
        }));
        authed.get('/agent/accounts', (req, res) => __awaiter(this, void 0, void 0, function* () {
            yield this.getAccounts(req, res);
        }));
        authed.post('/agent/profile', (new profileRequest_1.default(this)).validate(), (req, res) => __awaiter(this, void 0, void 0, function* () {
            yield this.updateProfile(req, res);
        }));
        authed.post('/agent/proxy', (new proxyRequest_1.default(this)).validate(), (req, res) => __awaiter(this, void 0, void 0, function* () {
            yield this.updateProxy(req, res);
        }));
        authed.post('/agent/proxy/reload', (req, res) => __awaiter(this, void 0, void 0, function* () {
            yield this.reloadProxy(req, res);
        }));
        authed.post('/agent/password', (new passwordRequest_1.default(this)).validate(), (req, res) => __awaiter(this, void 0, void 0, function* () {
            yield this.updatePassword(req, res);
        }));
        authed.post('/agent/lang', (new langRequest_1.default(this)).validate(), (req, res) => {
            this.updateLang(req, res);
        });
        authed.post('/event', (new eventRequest_1.default(this)).validate(), (req, res) => __awaiter(this, void 0, void 0, function* () {
            yield this.eventNotification(req, res);
        }));
        authed.get('/virtual_browsers', (req, res) => {
            this.getVirtualBrowsers(req, res);
        });
        authed.post('/virtual_browsers', (req, res) => __awaiter(this, void 0, void 0, function* () {
            yield this.startupVirtualBrowser(req, res);
        }));
        authed.post('/virtual_connector', (req, res) => __awaiter(this, void 0, void 0, function* () {
            yield this.startupVirtualConnector(req, res);
        }));
        authed.post('/file_relay', (req, res) => __awaiter(this, void 0, void 0, function* () {
            yield this.bootFileRelay(req, res);
        }));
        authed.post('/file_upload', (req, res) => __awaiter(this, void 0, void 0, function* () {
            yield this.fileUpload(req, res);
        }));
        authed.delete('/process', (req, res) => __awaiter(this, void 0, void 0, function* () {
            yield this.exitProcess(req, res);
        }));
        authed.post('/external_link', (req, res) => __awaiter(this, void 0, void 0, function* () {
            yield this.externalLink(req, res);
        }));
    }
    /**
     * APIレスポンス
     */
    apiResponse(res, statusCode, body = {}) {
        const headers = {
            'content-type': 'application/json; charset=utf-8',
        };
        body = Object.assign({}, { code: statusCode }, body);
        res.writeHead(statusCode, headers);
        res.write(JSON.stringify(body));
        res.end();
    }
    /**
     * HealthCheck
     */
    healthCheck(req, res) {
        this.apiResponse(res, 200, { message: 'ok' });
    }
    /**
     * エージェント関連情報
     */
    info(req, res) {
        var _a, _b, _c;
        return __awaiter(this, void 0, void 0, function* () {
            const agent = yield this.httpServer.app.getAgent();
            const payload = yield this.httpServer.app.getFullProfile();
            const lang = (yield this.httpServer.app.getLang()).value;
            try {
                const response = yield httpClient_1.default.post(`${config.appServer.endpoint}:${config.appServer.port}/api/agents/${agent.id}/info`, payload, {
                    headers: {
                        'content-type': 'application/json',
                        'content-language': lang,
                        'secret': agent.secret
                    }
                });
                yield this.httpServer.app.updateVersionInfo({
                    latestVersion: response.data.version_info.latest_version,
                    latestUrl: response.data.version_info.latest_url,
                    announcement: response.data.version_info.announcement
                });
                return this.apiResponse(res, 200, Object.assign({
                    'code': 200
                }, {
                    'version_info': this.httpServer.app.getVersionInfo(),
                    'agent_contracts': response.data.agent_contracts,
                    'agent_registration_requests': response.data.agent_registration_requests,
                    'expired': response.data.expired
                }));
            }
            catch (e) {
                yield this.httpServer.app.updateVersionInfo({
                    latestVersion: null,
                    latestUrl: null,
                    announcement: null
                });
                if ((_b = (_a = e.response) === null || _a === void 0 ? void 0 : _a.data) === null || _b === void 0 ? void 0 : _b.code) {
                    this.apiResponse(res, e.response.data.code, { message: e.response.data.message });
                }
                else {
                    if ((_c = e.response) === null || _c === void 0 ? void 0 : _c.status) {
                        this.apiResponse(res, e.response.status, { message: e.response.status.message });
                    }
                    else {
                        this.apiResponse(res, 500, { message: e.message });
                    }
                }
            }
        });
    }
    /**
     * エージェントバージョン確認
     */
    versionCheck(req, res) {
        var _a, _b, _c;
        return __awaiter(this, void 0, void 0, function* () {
            const payload = yield this.httpServer.app.getFullProfile();
            const lang = (yield this.httpServer.app.getLang()).value;
            try {
                const response = yield httpClient_1.default.post(`${config.appServer.endpoint}:${config.appServer.port}/api/agents/version`, payload, {
                    headers: {
                        'content-type': 'application/json',
                        'content-language': lang
                    }
                });
                const versionInfo = yield this.httpServer.app.updateVersionInfo({
                    latestVersion: response.data.latest_version,
                    latestUrl: response.data.latest_url,
                    announcement: response.data.announcement
                });
                return this.apiResponse(res, 200, Object.assign({
                    'code': 200
                }, versionInfo));
            }
            catch (e) {
                yield this.httpServer.app.updateVersionInfo({
                    latestVersion: null,
                    latestUrl: null,
                    announcement: null
                });
                if ((_b = (_a = e.response) === null || _a === void 0 ? void 0 : _a.data) === null || _b === void 0 ? void 0 : _b.code) {
                    this.apiResponse(res, e.response.data.code, { message: e.response.data.message });
                }
                else {
                    if ((_c = e.response) === null || _c === void 0 ? void 0 : _c.status) {
                        this.apiResponse(res, e.response.status, { message: e.response.status.message });
                    }
                    else {
                        this.apiResponse(res, 500, { message: e.message });
                    }
                }
            }
        });
    }
    /**
     * エージェント接続
     */
    connectAgent(req, res) {
        return __awaiter(this, void 0, void 0, function* () {
            const status = yield this.httpServer.app.connectAgent();
            this.apiResponse(res, 200, status);
        });
    }
    /**
     * エージェント切断
     */
    disconnectAgent(req, res) {
        return __awaiter(this, void 0, void 0, function* () {
            const status = yield this.httpServer.app.disconnectAgent();
            this.apiResponse(res, 200, status);
        });
    }
    /**
     * エージェントリセット
     */
    resetAgent(req, res) {
        return __awaiter(this, void 0, void 0, function* () {
            // エージェント情報をリセット
            this.httpServer.app.resetAgent();
            // UUIDを再発行
            const profile = yield this.httpServer.app.getProfile();
            profile.uuid = nodeUuid.v1();
            this.httpServer.app.setProfile(profile);
            // 切断
            yield this.disconnectAgent(req, res);
        });
    }
    /**
     * エージェント初期化
     */
    initializeAgent(req, res) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                yield this.httpServer.app.disconnectAgent(false);
                this.httpServer.app.clearHardware();
                this.httpServer.app.resetAgent();
                this.httpServer.app.deleteStatus();
                this.httpServer.app.deleteProfile();
                this.httpServer.app.deleteProxy();
                this.httpServer.app.deleteLang();
                this.httpServer.deleteCredentials();
                req.logout();
                req.session.destroy(() => {
                    this.apiResponse(res, 200);
                });
            }
            catch (e) {
                this.apiResponse(res, 500, { message: e.message });
            }
        });
    }
    /**
     * 登録リクエスト
     */
    registrationRequest(req, res) {
        var _a, _b;
        return __awaiter(this, void 0, void 0, function* () {
            const agent = yield this.httpServer.app.getAgent();
            if (agent.registered) {
                const lang = (yield this.httpServer.app.getLang()).value;
                try {
                    const response = yield httpClient_1.default.post(`${config.appServer.endpoint}:${config.appServer.port}/api/agents/${agent.id}/contracts`, {
                        'contract_member': req.body.user
                    }, {
                        'headers': {
                            'content-type': 'application/json',
                            'content-language': lang,
                            'secret': agent.secret
                        }
                    });
                    this.apiResponse(res, response.data.code);
                }
                catch (e) {
                    if ((_b = (_a = e.response) === null || _a === void 0 ? void 0 : _a.data) === null || _b === void 0 ? void 0 : _b.code) {
                        console.log(e.response.data);
                        this.apiResponse(res, e.response.data.code, { message: e.response.data.message });
                    }
                    else {
                        this.apiResponse(res, 500, { message: e.message });
                    }
                }
            }
            else {
                this.apiResponse(res, 412, {
                    message: 'Precondition Failed'
                });
            }
        });
    }
    /**
     * PINリクエスト
     */
    pinRequest(req, res) {
        var _a, _b;
        return __awaiter(this, void 0, void 0, function* () {
            const agent = yield this.httpServer.app.getAgent();
            if (agent.registered) {
                const lang = (yield this.httpServer.app.getLang()).value;
                try {
                    const response = yield httpClient_1.default.post(`${config.appServer.endpoint}:${config.appServer.port}/api/agents/${agent.id}/pins`, {
                        'pin': req.body.pin
                    }, {
                        'headers': {
                            'content-type': 'application/json',
                            'content-language': lang,
                            'secret': agent.secret
                        }
                    });
                    this.apiResponse(res, response.data.code);
                }
                catch (e) {
                    if ((_b = (_a = e.response) === null || _a === void 0 ? void 0 : _a.data) === null || _b === void 0 ? void 0 : _b.code) {
                        console.log(e.response.data);
                        this.apiResponse(res, e.response.data.code, { message: e.response.data.message });
                    }
                    else {
                        this.apiResponse(res, 500, { message: e.message });
                    }
                }
            }
            else {
                this.apiResponse(res, 412, {
                    message: 'Precondition Failed'
                });
            }
        });
    }
    /**
     * 登録リクエストをキャンセル
     */
    cancelRegistrationRequest(req, res) {
        var _a, _b;
        return __awaiter(this, void 0, void 0, function* () {
            const agent = yield this.httpServer.app.getAgent();
            if (agent.registered) {
                const lang = (yield this.httpServer.app.getLang()).value;
                try {
                    const response = yield httpClient_1.default.delete(`${config.appServer.endpoint}:${config.appServer.port}/api/agents/agent_registration_requests/${agent.id}`, {
                        user: req.body.user
                    }, {
                        'headers': {
                            'content-type': 'application/json',
                            'content-language': lang,
                            'secret': agent.secret
                        }
                    });
                    this.apiResponse(res, response.data.code);
                }
                catch (e) {
                    if ((_b = (_a = e.response) === null || _a === void 0 ? void 0 : _a.data) === null || _b === void 0 ? void 0 : _b.code) {
                        console.log(e.response.data);
                        this.apiResponse(res, e.response.data.code, { message: e.response.data.message });
                    }
                    else {
                        this.apiResponse(res, 500, { message: e.message });
                    }
                }
            }
            else {
                this.apiResponse(res, 412, {
                    message: 'Precondition Failed'
                });
            }
        });
    }
    /**
     * 登録を解除
     */
    releaseRegister(req, res) {
        var _a, _b;
        return __awaiter(this, void 0, void 0, function* () {
            const agent = yield this.httpServer.app.getAgent();
            if (agent.registered) {
                const lang = (yield this.httpServer.app.getLang()).value;
                try {
                    const response = yield httpClient_1.default.delete(`${config.appServer.endpoint}:${config.appServer.port}/api/agents/agent_contracts/${req.body.id}`, {}, {
                        'headers': {
                            'content-type': 'application/json',
                            'content-language': lang,
                            'secret': agent.secret
                        }
                    });
                    this.apiResponse(res, response.data.code);
                }
                catch (e) {
                    if ((_b = (_a = e.response) === null || _a === void 0 ? void 0 : _a.data) === null || _b === void 0 ? void 0 : _b.code) {
                        console.log(e.response.data);
                        this.apiResponse(res, e.response.data.code, { message: e.response.data.message });
                    }
                    else {
                        this.apiResponse(res, 500, { message: e.message });
                    }
                }
            }
            else {
                this.apiResponse(res, 412, {
                    message: 'Precondition Failed'
                });
            }
        });
    }
    /**
     * アカウントリスト取得
     */
    getAccounts(req, res) {
        var _a, _b, _c;
        return __awaiter(this, void 0, void 0, function* () {
            const agent = yield this.httpServer.app.getAgent();
            if (agent.registered) {
                const lang = (yield this.httpServer.app.getLang()).value;
                try {
                    const response = yield httpClient_1.default.get(`${config.appServer.endpoint}:${config.appServer.port}/api/agents/${agent.id}/contracts`, {
                        'headers': {
                            'content-type': 'application/json',
                            'content-language': lang,
                            'secret': agent.secret
                        }
                    });
                    this.apiResponse(res, response.data.code, response.data);
                }
                catch (e) {
                    if ((_b = (_a = e.response) === null || _a === void 0 ? void 0 : _a.data) === null || _b === void 0 ? void 0 : _b.code) {
                        console.log(e.response.data);
                        this.apiResponse(res, e.response.data.code, { message: e.response.data.message });
                    }
                    else {
                        if ((_c = e.response) === null || _c === void 0 ? void 0 : _c.status) {
                            this.apiResponse(res, e.response.status, { message: e.response.statusText });
                        }
                        else {
                            this.apiResponse(res, 500, { message: e.message });
                        }
                    }
                }
            }
            else {
                this.apiResponse(res, 412, {
                    message: 'Precondition Failed'
                });
            }
        });
    }
    /**
     * プロフィール更新
     */
    updateProfile(req, res) {
        return __awaiter(this, void 0, void 0, function* () {
            const agent = yield this.httpServer.app.getAgent();
            const profile = req.body;
            profile.uuid = (yield this.httpServer.app.getProfile()).uuid;
            profile.device_pixel_ratio = (yield this.httpServer.app.getProfile()).device_pixel_ratio;
            profile.max_values[1] = typeof profile.max_values[1] == 'string' ? parseInt(profile.max_values[1]) : profile.max_values[1];
            profile.max_values[2] = typeof profile.max_values[2] == 'string' ? parseInt(profile.max_values[2]) : profile.max_values[2];
            profile.registered = req.body.temp ? false : undefined;
            delete req.body.temp;
            const result = yield this.httpServer.app.notifyProfile(profile, agent);
            if (result.code == 200 || result.code == 401) {
                this.httpServer.app.setProfile(profile);
                yield this.httpServer.app.broadcastBoots();
                this.apiResponse(res, 200);
            }
            else {
                this.apiResponse(res, result.code, { message: result.message });
            }
        });
    }
    /**
     * プロキシ更新
     */
    updateProxy(req, res) {
        return __awaiter(this, void 0, void 0, function* () {
            const proxy = {
                enable: req.body.enable
            };
            if (!req.body.pac) {
                proxy.url = utils_1.createProxyUrl(req.body.host, req.body.port, req.body.userName, req.body.password);
            }
            else {
                proxy.pac = req.body.pac;
            }
            this.httpServer.app.setProxy(proxy);
            if ((yield this.httpServer.app.getStatus()).code >= 200) {
                yield this.httpServer.app.disconnectAgent();
                yield this.httpServer.app.connectAgent();
            }
            this.apiResponse(res, 200);
        });
    }
    /**
     * プロキシ設定をリロード
     */
    reloadProxy(req, res) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.httpServer.app.reloadProxy();
            if ((yield this.httpServer.app.getStatus()).code >= 200) {
                yield this.httpServer.app.disconnectAgent();
                yield this.httpServer.app.connectAgent();
            }
            this.apiResponse(res, 200);
        });
    }
    /**
     * パスワード更新
     */
    updatePassword(req, res) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.httpServer.setCredential('root', { secret: req.body.new_password });
            this.apiResponse(res, 200);
        });
    }
    /**
     * パスワード初期化
     */
    initPassword(req, res) {
        return __awaiter(this, void 0, void 0, function* () {
            if ((yield this.httpServer.getCredential('root')).secret === null) {
                yield this.httpServer.setCredential('root', { secret: req.body.new_password });
                req.flash('success', i18n_1.__('パスワードを初期化しました。'));
                this.apiResponse(res, 200);
            }
            else {
                this.apiResponse(res, 403, { message: 'Forbidden' });
            }
        });
    }
    /**
     * 言語更新
     */
    updateLang(req, res) {
        console.log(req.body.lang);
        this.httpServer.app.setLang(req.body.lang);
        req.flash('success', i18n_1.__('言語を更新しました。'));
        this.apiResponse(res, 200);
    }
    /**
     * 言語初期化
     */
    initLang(req, res) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!(yield this.httpServer.app.getLang()).defined) {
                this.httpServer.app.setLang(req.body.lang);
                req.flash('success', i18n_1.__('言語を更新しました。'));
                this.apiResponse(res, 200);
            }
            else {
                this.apiResponse(res, 403, { message: 'Forbidden' });
            }
        });
    }
    /**
     * イベント通知
     */
    eventNotification(req, res) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                yield this.httpServer.app.eventNotification(req.body.event, req.body.arguments);
                this.apiResponse(res, 200);
            }
            catch (e) {
                if (!e.code) {
                    e.code = 500;
                }
                this.apiResponse(res, e.code, { message: e.message });
            }
        });
    }
    /**
     * 仮想ブラウザリスト取得
     */
    getVirtualBrowsers(req, res) {
        console.log('getVirtualBrowsers');
        const browsers = this.httpServer.app.getBrowsers();
        const data = [];
        for (const uuid in browsers) {
            data.push({
                uuid: uuid
            });
        }
        this.apiResponse(res, 200, {
            browsers: data
        });
    }
    /**
     * 仮想ブラウザ起動
     */
    startupVirtualBrowser(req, res) {
        return __awaiter(this, void 0, void 0, function* () {
            console.log('startupVirtualBrowser');
            // 起動数上限の場合
            const browsers = this.httpServer.app.getBrowsers();
            const profile = yield this.httpServer.app.getProfile();
            if (utils_1.countKeys(browsers) >= profile.max_values[1]) {
                this.apiResponse(res, 200, { result: -1 });
                return;
            }
            const event = req.body;
            console.log(event);
            const browser = new browser_1.Browser('instance', this.httpServer.app.eventEmitter);
            try {
                yield browser.start(event);
                this.apiResponse(res, 200, { result: 0 });
            }
            catch (e) {
                this.apiResponse(res, 500, { result: -1, message: e.message });
                yield browser.stop();
            }
        });
    }
    /**
     * 仮想コネクタ起動
     */
    startupVirtualConnector(req, res) {
        return __awaiter(this, void 0, void 0, function* () {
            console.log('startupVirtualConnector');
            const connectorServerMigration = req.body;
            // 起動数上限の場合
            if (!connectorServerMigration.reconnect) {
                const connectors = this.httpServer.app.getConnectors();
                const profile = yield this.httpServer.app.getProfile();
                if (utils_1.countKeys(connectors) >= profile.max_values[2]) {
                    this.apiResponse(res, 200, { result: -1 });
                    return;
                }
            }
            const connector = new connector_1.Connector(this.httpServer.app.eventEmitter);
            try {
                yield connector.boot(connectorServerMigration);
                this.apiResponse(res, 200, { result: 0 });
            }
            catch (e) {
                this.apiResponse(res, 500, { result: -1, message: e.message });
            }
        });
    }
    /**
     * ファイル中継起動
     */
    bootFileRelay(req, res) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const fileRelayConnector = new connector_2.FileRelayConnector('instance', this.httpServer.app.getPort());
                const event = req.body;
                yield fileRelayConnector.connect(event);
                this.apiResponse(res, 200, { result: 0, tokenId: event.token.id });
            }
            catch (e) {
                this.apiResponse(res, 500, { result: -1, message: e.message });
            }
        });
    }
    /**
     * ファイルアップロード
     */
    fileUpload(req, res) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const event = req.body;
                const fileUploader = new uploader_1.FileUploader('instance', event);
                const tasks = yield fileUploader.preparate();
                fileUploader.upload(tasks); // アップロードは非同期で実行
                this.apiResponse(res, 200, { result: 0 });
            }
            catch (e) {
                console.log(e);
                this.apiResponse(res, 500, { result: -1, message: e.message });
            }
        });
    }
    /**
     * プロセス終了
     */
    exitProcess(req, res) {
        return __awaiter(this, void 0, void 0, function* () {
            this.apiResponse(res, 200);
            setTimeout(() => __awaiter(this, void 0, void 0, function* () {
                yield this.httpServer.app.exit();
            }), 0);
        });
    }
    /**
     * 外部リンク
     */
    externalLink(req, res) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.httpServer.app.externalLink(req.body.url)) {
                this.apiResponse(res, 200);
            }
            else {
                this.apiResponse(res, 200, { url: req.body.url });
            }
        });
    }
}
exports.API = API;
/**
* バリデートエラーレスポンス
*/
const validateErrorResponse = (req, res, next) => {
    const result = express_validator_1.validationResult(req);
    const errors = result.array();
    if (!errors.length) {
        next();
    }
    else {
        const headers = {
            'content-type': 'application/json; charset=utf-8',
        };
        const body = {
            code: 422,
            message: 'Validation error',
            errors: result.array().map((error) => {
                return error.msg;
            })
        };
        res.writeHead(422, headers);
        res.write(JSON.stringify(body));
        res.end();
    }
};
exports.validateErrorResponse = validateErrorResponse;
