"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const bcrypt = require("bcryptjs");
const express_validator_1 = require("express-validator");
const i18n_1 = require("i18n");
const api_1 = require("../api");
class default_1 {
    constructor(api) {
        this.api = api;
    }
    validate(initialize = false) {
        return [
            express_validator_1.check('current_password').custom((current_password) => __awaiter(this, void 0, void 0, function* () {
                if (!initialize) {
                    if (!current_password) {
                        throw new Error(i18n_1.__('%sは必須です。', i18n_1.__('現在のパスワード')));
                    }
                    if (!(yield bcrypt.compare(current_password, (yield this.api.httpServer.getCredential('root')).secret))) {
                        throw new Error(i18n_1.__('%sが一致しません。', i18n_1.__('現在のパスワード')));
                    }
                    if (current_password.length > 1024) {
                        throw new Error(i18n_1.__('%sは%d文字以下にしてください。', i18n_1.__('現在のパスワード'), '1024'));
                    }
                }
                return true;
            })),
            express_validator_1.check('new_password').custom((new_password, { req }) => {
                if (!new_password) {
                    throw new Error(i18n_1.__('%sは必須です。', i18n_1.__('新しいパスワード')));
                }
                if (new_password !== req.body.confirm_new_password) {
                    throw new Error(i18n_1.__('%sが一致しません。', i18n_1.__('新しいパスワード')));
                }
                if (new_password.length < 8) {
                    throw new Error(i18n_1.__('%sは%d文字以上にしてください。', i18n_1.__('新しいパスワード'), '8'));
                }
                if (new_password.length > 1024) {
                    throw new Error(i18n_1.__('%sは%d文字以下にしてください。', i18n_1.__('新しいパスワード'), '1024'));
                }
                return true;
            }),
            (req, res, next) => {
                api_1.validateErrorResponse(req, res, next);
            }
        ];
    }
}
exports.default = default_1;
