"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const express_validator_1 = require("express-validator");
const i18n_1 = require("i18n");
const api_1 = require("../api");
class default_1 {
    constructor(api) {
        this.api = api;
    }
    validate() {
        return [
            express_validator_1.body('enable').isBoolean().withMessage(() => i18n_1.__('%sはtrueかfalseにしてください。', i18n_1.__('有効'))),
            express_validator_1.body('host').custom((host, { req }) => {
                if (req.body.enable === true && !req.body.pac) {
                    if (!host) {
                        throw new Error(i18n_1.__('%sは必須です。', i18n_1.__('ホスト名')));
                    }
                    if (host.length > 1024) {
                        throw new Error(i18n_1.__('%sは%d文字以下にしてください。', i18n_1.__('ホスト名'), '1024'));
                    }
                }
                return true;
            }),
            express_validator_1.body('port').custom((port, { req }) => {
                if (req.body.enable === true && !req.body.pac) {
                    if (!port && port !== 0) {
                        throw new Error(i18n_1.__('%sは必須です。', i18n_1.__('ポート番号')));
                    }
                    if (typeof (port) !== 'number') {
                        throw new Error(i18n_1.__('%sは数字にしてください。', i18n_1.__('ポート番号')));
                    }
                    if (port < 0) {
                        throw new Error(i18n_1.__('%sは%d以上にしてください。', i18n_1.__('ポート番号'), '0'));
                    }
                    if (port > 65535) {
                        throw new Error(i18n_1.__('%sは%d以下にしてください。', i18n_1.__('ポート番号'), '65535'));
                    }
                }
                return true;
            }),
            express_validator_1.body('userName').custom((userName, { req }) => {
                if (req.body.enable === true && !req.body.pac) {
                    if (req.body.password) {
                        if (!userName) {
                            throw new Error(i18n_1.__('%sが入力されている場合、%sは必須です。', i18n_1.__('パスワード'), i18n_1.__('ユーザ名')));
                        }
                    }
                    if ((userName === null || userName === void 0 ? void 0 : userName.length) > 1024) {
                        throw new Error(i18n_1.__('%sは%d文字以下にしてください。', i18n_1.__('ユーザ名'), '1024'));
                    }
                }
                return true;
            }),
            express_validator_1.body('password').custom((password, { req }) => {
                if (req.body.enable === true && !req.body.pac && req.body.userName) {
                    if (password.length > 1024) {
                        throw new Error(i18n_1.__('%sは%d文字以下にしてください。', i18n_1.__('パスワード'), '1024'));
                    }
                }
                return true;
            }),
            express_validator_1.body('pac').optional({ nullable: true }).isURL({ protocols: ['http', 'https'], require_protocol: true, require_tld: false }).withMessage(() => i18n_1.__('%sの書式が正しくありません。', 'URL')),
            (req, res, next) => {
                api_1.validateErrorResponse(req, res, next);
            }
        ];
    }
}
exports.default = default_1;
