"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.SocketIoServer = void 0;
const io = require("socket.io");
const utils_1 = require("../../utils");
class SocketIoServer {
    constructor(httpServer) {
        this.io = new io(httpServer.server, {
            pingTimeout: 15000,
            pingInterval: 5000
        });
        this.httpServer = httpServer;
        this.sockets = new Set;
        // セッションを共有
        this.io.use(function (socket, next) {
            httpServer.session(socket.request, socket.request.res, next);
        });
    }
    preparate() {
        this.io.on('connection', (socket) => __awaiter(this, void 0, void 0, function* () {
            socket.on('disconnect', () => {
                console.log('socket.io disconnect');
                this.sockets.delete(socket);
            });
            if (socket.request.headers['x-csrf-token'] === undefined || socket.request.session.csrfToken !== socket.request.headers['x-csrf-token']) {
                socket.disconnect();
            }
            else {
                console.log('socket.io connection');
                this.sockets.add(socket);
                this.emitStatus(socket, yield this.httpServer.app.getStatus());
                this.emitBoots(socket, {
                    browser: {
                        current: utils_1.countKeys(this.httpServer.app.getBrowsers()),
                        max: (yield this.httpServer.app.getProfile()).max_values[1]
                    },
                    connector: {
                        current: utils_1.countKeys(this.httpServer.app.getConnectors()),
                        max: (yield this.httpServer.app.getProfile()).max_values[2]
                    }
                });
            }
        }));
    }
    /**
     * ステータスをブロードキャスト
     */
    broadcastStatus() {
        return __awaiter(this, void 0, void 0, function* () {
            for (const socket of this.sockets) {
                this.emitStatus(socket, yield this.httpServer.app.getStatus());
            }
        });
    }
    /**
     * ステータスをエミット
     */
    emitStatus(socket, status) {
        socket.emit('statusNotification', status);
    }
    /**
     * 起動数をブロードキャスト
     */
    broadcastBoots() {
        return __awaiter(this, void 0, void 0, function* () {
            const boots = {
                browser: {
                    current: utils_1.countKeys(this.httpServer.app.getBrowsers()),
                    max: (yield this.httpServer.app.getProfile()).max_values[1]
                },
                connector: {
                    current: utils_1.countKeys(this.httpServer.app.getConnectors()),
                    max: (yield this.httpServer.app.getProfile()).max_values[2]
                }
            };
            for (const socket of this.sockets) {
                this.emitBoots(socket, boots);
            }
        });
    }
    /**
     * 起動数をエミット
     */
    emitBoots(socket, boots) {
        socket.emit('bootsNotification', boots);
    }
}
exports.SocketIoServer = SocketIoServer;
