"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ClientSocket = void 0;
const events_1 = require("events");
class ClientSocket extends events_1.EventEmitter {
    constructor(id, socketIoRelay) {
        super();
        this._id = id;
        this.socketIoRelay = socketIoRelay;
        this.events = [];
        this._disconnected = false;
    }
    get id() {
        return this._id;
    }
    get disconnected() {
        return this._disconnected;
    }
    get serverSocket() {
        return this.socketIoRelay;
    }
    set disconnected(v) {
        this._disconnected = v;
    }
    clientEvent(event) {
        return this._id + '/' + event;
    }
    isSocketRelayEvent(event) {
        if (event === 'disconnect') {
            return false;
        }
        else {
            return true;
        }
    }
    on(event, listener) {
        if (this.isSocketRelayEvent(event)) {
            const cEvent = this.clientEvent(event);
            const socket = this.socketIoRelay.getSocket();
            socket.emit('attach', cEvent);
            socket.on(cEvent, listener);
            this.events.push(cEvent);
        }
        else {
            super.on(event, listener);
        }
        return this;
    }
    once(event, listener) {
        if (this.isSocketRelayEvent(event)) {
            const cEvent = this.clientEvent(event);
            const socket = this.socketIoRelay.getSocket();
            socket.emit('attach', cEvent);
            socket.once(cEvent, listener);
        }
        else {
            super.once(event, listener);
        }
        return this;
    }
    off(event, listener) {
        if (this.isSocketRelayEvent(event)) {
            const cEvent = this.clientEvent(event);
            const socket = this.socketIoRelay.getSocket();
            socket.off(cEvent, listener);
        }
        else {
            super.off(event, listener);
        }
        return this;
    }
    emit(event, ...args) {
        if (typeof (event) === 'string') {
            if (this.isSocketRelayEvent(event)) {
                this.socketIoRelay.send(this._id, event, ...args);
            }
            else {
                super.emit(event, ...args);
            }
            return true;
        }
        return false;
    }
    removeAllSocketListeners() {
        const socket = this.socketIoRelay.getSocket();
        this.events.forEach((event) => {
            socket.removeEventListener(event);
        });
    }
    disconnect() {
        const socket = this.socketIoRelay.getSocket();
        socket.emit('client_disconnect', this._id);
    }
}
exports.ClientSocket = ClientSocket;
