"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const moment = require("moment");
class RelayTokenStore {
    constructor() {
        this.tokens = {};
        this.timers = {};
    }
    put(token) {
        token.expire = moment().add(token.timeout, 'seconds').valueOf();
        this.tokens[token.id] = token;
        this.timers[token.id] = setTimeout(() => {
            this.delete(token.id);
        }, token.timeout);
    }
    get(id) {
        if (this.tokens[id]) {
            const token = this.tokens[id];
            this.delete(id);
            return token;
        }
        return null;
    }
    delete(id) {
        clearTimeout(this.timers[id]);
        delete this.timers[id];
        delete this.tokens[id];
    }
    authRequest(req, res, next) {
        const id = req.header('x-api-relay-token');
        if (!id) {
            res.status(403).send();
            return;
        }
        const token = this.get(id);
        if (!token) {
            res.status(403).send();
            return;
        }
        const method = req.method;
        const path = `${req.baseUrl}${req.url}`;
        if (token.method != method || token.path != decodeURI(path)) {
            res.status(403).send();
            return;
        }
        next();
    }
}
exports.default = new RelayTokenStore;
