"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.SocketIoRelay = void 0;
const socketIo = require("socket.io-client");
const ClientSocket_1 = require("./ClientSocket");
const events_1 = require("events");
class SocketIoRelay extends events_1.EventEmitter {
    constructor() {
        super();
        this.socket = {};
        this.url = null;
        this.clients = {};
        this.namespaces = {};
    }
    get serverId() {
        return this.server_id;
    }
    get relayEndpoint() {
        return this.relay.endpoint;
    }
    get relayPort() {
        return this.relay.port;
    }
    get relayId() {
        return this.relay.id;
    }
    get relayNamespace() {
        return this.relay.namespace;
    }
    get disconnected() {
        return this.socket.disconnected;
    }
    listen(relay, serverId) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.startListen(relay, serverId, null);
        });
    }
    startListen(relay, serverId, socket) {
        const _super = Object.create(null, {
            emit: { get: () => super.emit }
        });
        return __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => {
                var _a;
                this.url = `${relay.endpoint}:${relay.port}`;
                if (!relay.switchId) {
                    relay.switchId = null;
                }
                this.relay = relay;
                if (socket == null) {
                    this.socket = socketIo.connect(this.url, {
                        reconnection: false,
                        forceNew: true,
                        query: {
                            relay_id: relay.id
                        },
                        agent: (_a = relay.agent) !== null && _a !== void 0 ? _a : null
                    });
                }
                else {
                    this.socket = socket;
                }
                this.server_id = serverId;
                this.socket.emit('listen', serverId, relay.switchId, (relay) => {
                    this.relay.id = relay.id;
                    this.relay.namespace = relay.namespace;
                    console.log('Listening from %s', relay.namespace);
                    resolve();
                });
                this.socket.on('client_connect', (clientId, _serverId, headers) => {
                    if (serverId == _serverId) {
                        // 自分のネームスペースのみを処理する
                        console.log('Connected ws client[#%s] from server [#%s]', clientId, serverId);
                        const clientSocket = new ClientSocket_1.ClientSocket(clientId, this, headers);
                        this.clients[clientId] = clientSocket;
                        _super.emit.call(this, 'connect', clientSocket);
                    }
                });
                this.socket.on('client_disconnect', (clientId, _serverId) => {
                    if (serverId == _serverId) {
                        // 自分のネームスペースのみを処理する
                        if (clientId in this.clients) {
                            console.log('Disconnected ws client[#%s] from server [#%s]', clientId, serverId);
                            this.clients[clientId].removeAllSocketListeners();
                            this.clients[clientId].emit('disconnect', clientId);
                            this.clients[clientId].disconnected = true;
                            delete this.clients[clientId];
                        }
                    }
                });
                this.socket.on('connect', () => {
                    console.log('Connected to relay server(#%s)', serverId);
                    _super.emit.call(this, 'relay_connect');
                });
                this.socket.on('connect_error', () => {
                    console.log('Connect error to relay server(#%s)', serverId);
                    _super.emit.call(this, 'relay_connect_error');
                    reject(new Error('Relay connect error'));
                });
                this.socket.on('disconnect', () => {
                    console.log('Disconnected from relay server(#%s)', serverId);
                    for (const clientId in this.clients) {
                        console.log('Disconnected ws client[#%s]', clientId);
                        this.clients[clientId].removeAllSocketListeners();
                        this.clients[clientId].emit('disconnect', clientId);
                        this.clients[clientId].disconnected = true;
                        delete this.clients[clientId];
                    }
                    _super.emit.call(this, 'relay_disconnect');
                });
            });
        });
    }
    of(namespace) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.url === null) {
                return null;
            }
            if (namespace in this.namespaces) {
                return this.namespaces[namespace];
            }
            const newSocketIoRelay = new SocketIoRelay();
            const newServerId = this.serverId + '/' + namespace;
            yield newSocketIoRelay.startListen(this.relay, newServerId, this.socket);
            this.namespaces[namespace] = newSocketIoRelay;
            return newSocketIoRelay;
        });
    }
    on(event, listener) {
        super.on(event, listener);
        return this;
    }
    once(event, listener) {
        super.once(event, listener);
        return this;
    }
    off(event, listener) {
        super.off(event, listener);
        return this;
    }
    emit(event, ...datas) {
        if (typeof (event) === 'string') {
            this.send(null, event, ...datas);
            return true;
        }
        return false;
    }
    send(clientId, eventName, ...datas) {
        const emitData = {
            clientId,
            serverId: this.serverId,
            eventName,
        };
        this.socket.emit('emit', emitData, ...datas);
    }
    getSocket() {
        return this.socket;
    }
    close() {
        this.socket.close();
    }
    startSwitch(switchId, serverIds = []) {
        return new Promise((resolve, reject) => {
            this.socket.emit('start_switch', { switchId: switchId, serverIds: [this.server_id, ...serverIds] }, (result) => {
                resolve(result);
            });
        });
    }
    endSwitch(serverIds = []) {
        return new Promise((resolve, reject) => {
            this.socket.emit('end_switch', { serverIds: [this.server_id, ...serverIds] }, (result) => {
                resolve(result);
            });
        });
    }
    reconnectClients() {
        return new Promise((resolve, reject) => {
            this.socket.emit('reconnect_clients', { serverId: this.server_id }, (result) => {
                resolve(result);
            });
        });
    }
    startBuffer(eventName) {
        return new Promise((resolve, reject) => {
            this.socket.emit('start_buffer', { serverId: this.server_id, eventName: eventName }, (result) => {
                resolve(result);
            });
        });
    }
}
exports.SocketIoRelay = SocketIoRelay;
