"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Connector = void 0;
const config = require("config");
const events = require("events");
const httpClient_1 = require("../../httpClient");
const net = require("net");
const SocketIOTunnel = require("socket.io-tunnel-stream");
const utils_1 = require("./../../utils");
class Connector extends events {
    constructor() {
        super(...arguments);
        this.tcpServer = null;
        this.listenPort = null;
    }
    get connected() {
        var _a;
        return !!((_a = this.socket) === null || _a === void 0 ? void 0 : _a.connected);
    }
    connect(connectorInfo, callback) {
        return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
            this.connectorInfo = connectorInfo;
            const endpoint = `${this.connectorInfo.server.endpoint}:${this.connectorInfo.server.port}${this.connectorInfo.server.path ? this.connectorInfo.server.path : ''}`;
            /**
             * オプション設定
             */
            const options = {
                reconnection: false,
                forceNew: true,
                query: {
                    relay_id: this.connectorInfo.server.relayId,
                },
                timeout: 10000
            };
            options.agent = yield httpClient_1.default.getHttpAgentForSocketIo(`${this.connectorInfo.server.endpoint}:${this.connectorInfo.server.port}/socket.io/socket.io.js?relay_id=${this.connectorInfo.server.relayId}`);
            this.socket = utils_1.startClientSocket(endpoint, options);
            this.tunnelBuilder = new SocketIOTunnel.TunnelBuilder(this.socket);
            this.socket.on('connect', () => __awaiter(this, void 0, void 0, function* () {
                try {
                    yield this.socket.waitUntilCanPingPong();
                    if (!this.connectorInfo.proxy.host) {
                        const proxy = yield this.getProxy();
                        this.connectorInfo.proxy.host = proxy.host;
                        this.connectorInfo.proxy.port = proxy.port;
                    }
                    const port = yield this.attemptBuildTunnel();
                    if (callback) {
                        yield callback();
                    }
                    resolve(port);
                }
                catch (e) {
                    this.socket.disconnect();
                    reject(e);
                }
            }));
            this.socket.on('connect_error', (err) => __awaiter(this, void 0, void 0, function* () {
                console.log(`connect_error. [${err.message}]`);
                reject(new Error('connect error'));
            }));
            this.socket.on('disconnect', () => __awaiter(this, void 0, void 0, function* () {
                if (this.connectorInfo.reconnection) {
                    this.emit('reconnect');
                }
                else {
                    this.emit('disconnect');
                }
            }));
        }));
    }
    disconnect() {
        var _a;
        (_a = this.socket) === null || _a === void 0 ? void 0 : _a.close();
        this.tcpServer.close();
        this.socket = null;
        this.tunnelBuilder = null;
        this.tcpServer = null;
        this.listenPort = null;
    }
    attemptBuildTunnel() {
        return __awaiter(this, void 0, void 0, function* () {
            let [startPort, endPort] = config.port.connectorForward.split('-').map((port) => {
                return parseInt(port);
            });
            if (!endPort) {
                endPort = startPort;
            }
            const attemptListen = (port) => {
                return new Promise((resolve, reject) => {
                    const _attemptListen = (port) => {
                        const listeningHandler = () => {
                            tcpServer.off('error', errorHandler);
                            resolve({ tcpServer: tcpServer, listenPort: port });
                        };
                        const errorHandler = (err) => {
                            tcpServer.off('listening', listeningHandler);
                            tcpServer.close();
                            console.log(err.message);
                            if (err.message.match('EADDRINUSE') && port < endPort) {
                                _attemptListen(++port);
                            }
                            else {
                                reject(err);
                            }
                        };
                        const tcpServer = net.createServer();
                        tcpServer.on('listening', listeningHandler);
                        tcpServer.on('error', errorHandler);
                        tcpServer.listen(port);
                    };
                    _attemptListen(port);
                });
            };
            const attemptTunnel = (tcpServer) => {
                return new Promise((resolve, reject) => {
                    this.tunnelBuilder.on('error_tunnel', (error) => {
                        tcpServer.close();
                        reject(new Error(error));
                    });
                    this.tunnelBuilder.emit('connector_tunnel', {
                        tcpHost: this.connectorInfo.proxy.host,
                        tcpPort: this.connectorInfo.proxy.port,
                        listenServer: tcpServer
                    }, (tunnel) => {
                        if (tunnel) {
                            resolve();
                        }
                    });
                });
            };
            if (!this.tcpServer) {
                const { tcpServer, listenPort } = yield attemptListen(startPort);
                yield attemptTunnel(tcpServer);
                this.tcpServer = tcpServer;
                this.listenPort = listenPort;
            }
            else {
                yield attemptTunnel(this.tcpServer);
            }
            console.log(`connector_tunnel established. [listenPort: ${this.listenPort}]`);
            return this.listenPort;
        });
    }
    getProxy() {
        return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
            try {
                yield this.socket.emitWithAckTimeout('web_proxy', (err, proxy) => {
                    if (err) {
                        reject(new Error(err.message));
                    }
                    else {
                        resolve(proxy);
                    }
                });
            }
            catch (e) {
                reject(e);
            }
        }));
    }
}
exports.Connector = Connector;
