"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.DownloadWatcher = void 0;
const chokidar = require("chokidar");
const config = require("config");
const events = require("events");
const fs = require("fs");
const path = require("path");
class DownloadWatcher extends events {
    constructor(uuid, contractId) {
        super();
        this.files = [];
        this.updateLocks = {};
        this.uuid = uuid;
        this.pathWithoutContractId = `downloads/${uuid}`;
        this.path = `${contractId}/${this.pathWithoutContractId}`;
        fs.mkdirSync(`${config.storage.path}/${this.path}`, {
            recursive: true
        });
        this.watcher = chokidar.watch(`${config.storage.path}/${this.path}`, {
            usePolling: true,
            interval: 1000,
            binaryInterval: 1000
        });
        this.watcher.on('add', (file) => __awaiter(this, void 0, void 0, function* () {
            if (fs.existsSync(file) && file.indexOf(`${this.uuid}.lock`) === -1) {
                yield this.addFile(file);
                this.notification();
            }
        }));
        this.watcher.on('change', (file) => {
            if (!this.updateLocks[file]) {
                this.updateLocks[file] = setTimeout(() => __awaiter(this, void 0, void 0, function* () {
                    delete this.updateLocks[file];
                    if (fs.existsSync(file)) {
                        yield this.updateFile(file);
                        this.notification();
                    }
                }), 1000);
            }
        });
        this.watcher.on('unlink', (file) => __awaiter(this, void 0, void 0, function* () {
            yield this.unlinkFile(file);
            this.notification();
        }));
    }
    notification() {
        return __awaiter(this, void 0, void 0, function* () {
            clearTimeout(this.notificationTimer);
            this.notificationTimer = setTimeout(() => {
                this.files.sort((a, b) => {
                    return a.createdAt > b.createdAt ? -1 : 1;
                });
                this.emit('files', this.files);
            }, 100);
        });
    }
    addFile(file) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const stats = yield fs.promises.stat(file);
                const formatedStats = yield this.formatStats(file, stats);
                this.files.push(formatedStats);
            }
            catch (e) {
                console.log('downloadWatcher: addFile error', e.message);
            }
        });
    }
    updateFile(file) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const stats = yield fs.promises.stat(file);
                const parsedFile = path.parse(file);
                const index = this.files.findIndex((file) => {
                    return file.name == `${this.pathWithoutContractId}/${parsedFile.name}${parsedFile.ext}`;
                });
                if (index >= 0) {
                    const formatedStats = yield this.formatStats(file, stats);
                    this.files[index] = formatedStats;
                }
            }
            catch (e) {
                console.log('downloadWatcher: updateFile error', e.message);
            }
        });
    }
    unlinkFile(file) {
        return __awaiter(this, void 0, void 0, function* () {
            const parsedFile = path.parse(file);
            const index = this.files.findIndex((file) => {
                return file.name == `${this.pathWithoutContractId}/${parsedFile.name}${parsedFile.ext}`;
            });
            if (index >= 0) {
                this.files.splice(index, 1);
            }
        });
    }
    formatStats(file, stats) {
        return __awaiter(this, void 0, void 0, function* () {
            const parsedFile = path.parse(file);
            return {
                name: `${this.pathWithoutContractId}/${parsedFile.name}${parsedFile.ext}`,
                size: stats.size,
                createdAt: stats.mtimeMs,
                ino: stats.ino
            };
        });
    }
    close() {
        return __awaiter(this, void 0, void 0, function* () {
            this.removeAllListeners();
            yield this.watcher.close();
        });
    }
}
exports.DownloadWatcher = DownloadWatcher;
