"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.FileManager = void 0;
const config = require("config");
const events = require("events");
const fs = require("fs");
const cbot_api_client_1 = require("cbot-api-client");
const httpClient_1 = require("../../httpClient");
const utils_1 = require("../../utils");
class FileManager extends events {
    constructor(tokenId, contractId) {
        super();
        this.tokenId = tokenId;
        this.contractId = contractId;
    }
    get(targets) {
        return __awaiter(this, void 0, void 0, function* () {
            const files = [];
            const total = targets.reduce((total, target) => {
                var _a;
                return total + ((_a = target.meta) === null || _a === void 0 ? void 0 : _a.size);
            }, 0);
            let progress = 0;
            let notifyLock = false;
            if (this.tokenId) {
                yield this.connect();
            }
            try {
                for (const target of targets) {
                    if (target.ref.indexOf('.') !== -1) {
                        console.log('Invalid ref:', target.ref);
                        throw new Error('Invalid ref');
                    }
                    let path = `${this.contractId}/`;
                    if (target.ref.indexOf('copied/') === 0) {
                        path += target.ref;
                    }
                    else {
                        path += `cache/${target.ref}`;
                    }
                    // すでにファイルが存在している場合はそのまま再利用する
                    if (fs.existsSync(`${config.storage.path}/${path}`)) {
                        const fullPath = `${config.storage.path}/${path}`;
                        try {
                            const currentTime = new Date();
                            const filename = (yield fs.promises.readdir(fullPath))[0];
                            fs.utimesSync(`${fullPath}/${filename}`, currentTime, currentTime);
                            files.push(`${fullPath}/${filename}`);
                            continue;
                        }
                        catch (e) {
                            // ファイルが存在していない
                            console.log(e.message);
                        }
                    }
                    // ローカルファイルの場合でファイルが存在しない場合はエラー
                    else if (target.ref.indexOf('copied/') === 0) {
                        throw new Error('Invalid ref');
                    }
                    // ファイルが存在しない場合はダウンロードする
                    const { stream, meta } = yield this.apiClient.ws.download(target.ref);
                    let writeStream;
                    const fullPath = yield utils_1.makeStorageDirAndTryCallback(`${path}.download`, (fullPath) => __awaiter(this, void 0, void 0, function* () {
                        writeStream = fs.createWriteStream(`${fullPath}/${meta.filename}`);
                        return fullPath;
                    }));
                    stream.pipe(writeStream);
                    yield new Promise((resolve, reject) => {
                        stream.on('data', (chunk) => {
                            progress += chunk.length;
                            if (!notifyLock || progress >= total) {
                                this.emit('progress', {
                                    percent: progress / total * 100
                                });
                                if (progress < total) {
                                    notifyLock = true;
                                    setTimeout(() => {
                                        notifyLock = false;
                                    }, 1000);
                                }
                            }
                        });
                        stream.on('end', () => {
                            resolve();
                        });
                        stream.on('error', (err) => __awaiter(this, void 0, void 0, function* () {
                            yield fs.promises.rmdir(fullPath, {
                                recursive: true
                            });
                            reject(err);
                        }));
                    });
                    const movedFullPath = fullPath.replace(/\.download$/, '');
                    yield fs.promises.rename(fullPath, movedFullPath);
                    files.push(`${movedFullPath}/${meta.filename}`);
                }
            }
            catch (e) {
                throw e;
            }
            finally {
                if (this.tokenId) {
                    this.disconnect();
                }
                this.removeAllListeners();
            }
            return files;
        });
    }
    connect() {
        return __awaiter(this, void 0, void 0, function* () {
            const url = `${config.apiServer.endpoint}:${config.apiServer.port}`;
            const httpAgent = yield httpClient_1.default.getHttpAgentForSocketIo(`${url}/socket.io/socket.io.js`);
            this.apiClient = new cbot_api_client_1.default({
                lang: 'en',
                endpoint: url,
                agent: httpAgent
            });
            yield this.apiClient.ws.connect(this.tokenId);
        });
    }
    disconnect() {
        this.apiClient.ws.disconnect();
    }
}
exports.FileManager = FileManager;
