"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Connector = void 0;
const httpClient_1 = require("../../httpClient");
const SocketIOTunnel = require("socket.io-tunnel-stream");
const socket_io_rawRelay_1 = require("../../socket-io.rawRelay");
const webProxy_1 = require("./webProxy");
class Connector {
    constructor(eventEmitter) {
        this.socket = null;
        this.tunnelBuilder = null;
        this.eventEmitter = eventEmitter;
    }
    /**
     * ID取得
     */
    getId() {
        return this.migrationInfo.id;
    }
    /**
     * コネクタ起動
     */
    boot(migrationInfo) {
        return __awaiter(this, void 0, void 0, function* () {
            this.migrationInfo = migrationInfo;
            yield this.startConnectorNotification();
            try {
                yield this.migrate(migrationInfo);
            }
            catch (e) {
                yield this.stopConnectorNotification();
                throw e;
            }
            this.setEvents();
        });
    }
    /**
     * コネクタ起動通知
     */
    startConnectorNotification() {
        return new Promise((resolve, reject) => {
            // コネクタ起動通知
            this.eventEmitter.emit('start_connector', this, () => {
                resolve();
            });
        });
    }
    /**
     * コネクタ停止通知
     */
    stopConnectorNotification() {
        return new Promise((resolve, reject) => {
            // コネクタ停止通知
            this.eventEmitter.emit('stop_connector', this, () => {
                resolve();
            });
        });
    }
    /**
     * トンネルサーバをマイグレーション
     */
    migrate(migrationInfo) {
        return __awaiter(this, void 0, void 0, function* () {
            this.socket = new socket_io_rawRelay_1.SocketIoRelay();
            const httpAgent = yield httpClient_1.default.getHttpAgentForSocketIo(`${migrationInfo.relay.endpoint}:${migrationInfo.relay.port}/socket.io/socket.io.js?relay_id=${migrationInfo.relay.id}`);
            try {
                yield this.socket.listen({
                    endpoint: migrationInfo.relay.endpoint,
                    port: migrationInfo.relay.port,
                    switchId: migrationInfo.relay.switchId,
                    id: migrationInfo.relay.id,
                    agent: httpAgent
                }, migrationInfo.id);
                if (!(yield this.socket.endSwitch())) {
                    throw new Error('End switch failed');
                }
            }
            catch (e) {
                this.socket.close();
                throw e;
            }
        });
    }
    /**
     * イベント設定
     */
    setEvents() {
        const close = () => __awaiter(this, void 0, void 0, function* () {
            this.socket.close();
            this.socket = null;
            yield this.stopConnectorNotification();
        });
        const timeout = setTimeout(() => __awaiter(this, void 0, void 0, function* () {
            yield close();
        }), 30000);
        this.socket.on('connect', (socket) => {
            console.log('Connected to WS relay server.');
            clearTimeout(timeout);
            this.tunnelBuilder = new SocketIOTunnel.TunnelBuilder(socket);
            socket.on('disconnect', () => __awaiter(this, void 0, void 0, function* () {
                this.tunnelBuilder = null;
                yield webProxy_1.default.stop(socket.id);
                setTimeout(() => __awaiter(this, void 0, void 0, function* () {
                    yield close();
                }));
            }));
            socket.on('web_proxy', (ack) => __awaiter(this, void 0, void 0, function* () {
                try {
                    const proxy = yield webProxy_1.default.start(socket.id);
                    ack(null, proxy);
                }
                catch (e) {
                    ack({ message: e.message });
                }
            }));
            socket.on('_ping', () => {
                socket.emit('_pong');
            });
        });
    }
}
exports.Connector = Connector;
