"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.RestClient = void 0;
const axios_1 = require("axios");
class RestClient {
    constructor(options) {
        this.accessToken = null;
        this.secretKey = null;
        this.oauthAccessToken = null;
        this.options = options;
        this.setLang(options.lang);
        if (this.options.agent === undefined) {
            this.options.agent = false;
        }
    }
    setLang(lang) {
        this.options.lang = lang;
    }
    setAccessToken(accessToken, secretKey) {
        this.accessToken = accessToken;
        this.secretKey = secretKey;
        this.oauthAccessToken = null;
    }
    setOAuthAccessToken(accessToken) {
        this.oauthAccessToken = accessToken;
        this.accessToken = null;
        this.secretKey = null;
    }
    get(path) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield axios_1.default.get(this.getRequestUrl(path), {
                headers: this.getRequestHeaders(),
                httpAgent: this.options.agent,
                httpsAgent: this.options.agent
            });
        });
    }
    post(path, payload) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield axios_1.default.post(this.getRequestUrl(path), payload, {
                headers: this.getRequestHeaders(),
                httpAgent: this.options.agent,
                httpsAgent: this.options.agent
            });
        });
    }
    delete(path, payload) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield axios_1.default.delete(this.getRequestUrl(path), {
                headers: this.getRequestHeaders(),
                httpAgent: this.options.agent,
                httpsAgent: this.options.agent,
                data: payload
            });
        });
    }
    getRequestHeaders() {
        const headers = {
            'x-cbot-content-type': 'application/json',
            'x-cbot-content-language': this.options.lang
        };
        if (this.accessToken) {
            headers["x-cbot-access-token"] = this.accessToken;
            headers["x-cbot-secret-key"] = this.secretKey;
        }
        else if (this.oauthAccessToken) {
            headers["x-cbot-authorization"] = `Bearer ${this.oauthAccessToken}`;
        }
        return headers;
    }
    getRequestUrl(path) {
        if (path.indexOf('/') !== 0) {
            path = `/${path}`;
        }
        return `${this.options.endpoint}${path}`;
    }
}
exports.RestClient = RestClient;
//# sourceMappingURL=rest.js.map