"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.WSClient = void 0;
const io = require("socket.io-client");
const ss = require("socket.io-stream");
class WSClient {
    constructor(options) {
        this.options = options;
    }
    connect(tokenId) {
        return new Promise((resolve, reject) => {
            this.socket = io.connect(this.options.endpoint, {
                timeout: 10000,
                reconnectionAttempts: 6,
                transports: ["websocket"],
                agent: this.options.agent
            });
            this.socket.on('connect', () => __awaiter(this, void 0, void 0, function* () {
                console.log('API-Client connected');
                try {
                    yield this.authorize(tokenId);
                    resolve();
                }
                catch (e) {
                    this.socket.disconnect();
                    reject(e);
                }
            }));
            this.socket.on('disconnect', () => {
                console.log('API-Client disconnect');
            });
            this.socket.on('connect_error', (err) => {
                reject(err);
            });
        });
    }
    disconnect() {
        var _a;
        (_a = this.socket) === null || _a === void 0 ? void 0 : _a.close();
    }
    upload(readStream, params) {
        return new Promise((resolve, reject) => {
            const writeStream = ss.createStream();
            ss(this.socket).emit('upload', writeStream, params, (err, ref) => {
                if (err) {
                    writeStream.destroy();
                    reject(new Error(err.message));
                }
                else {
                    resolve(ref);
                }
            });
            readStream.pipe(writeStream);
        });
    }
    download(ref) {
        return new Promise((resolve, reject) => {
            ss(this.socket).emit('download', ref, (err, stream, meta) => {
                if (err) {
                    reject(new Error(err.message));
                }
                else {
                    resolve({
                        stream: stream,
                        meta: meta
                    });
                }
            });
        });
    }
    authorize(tokenId) {
        return new Promise((resolve, reject) => {
            this.socket.emit('authorize', tokenId, (err) => {
                if (err) {
                    reject(new Error(err.message));
                }
                else {
                    resolve();
                }
            });
        });
    }
}
exports.WSClient = WSClient;
//# sourceMappingURL=ws.js.map