"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.parseChallenge = exports.Challenge = void 0;
const crypto_1 = __importDefault(require("crypto"));
const url_1 = __importDefault(require("url"));
class Challenge {
    constructor(challenge, proxy) {
        this.proxy = proxy;
        this.nc = 0;
        this.data = parseChallenge(challenge);
    }
    /**
     * ダイジェスト生成
     */
    createDigest(method, uri) {
        const md5 = (str) => {
            const md5 = crypto_1.default.createHash('md5');
            return md5.update(str, 'binary').digest('hex');
        };
        const cnonce = md5(String(new Date().getTime()));
        const parsed = url_1.default.parse(uri);
        const path = parsed.path;
        const nc = this.getNc();
        let username = '', password = '';
        if (this.proxy.auth) {
            [username, password] = this.proxy.auth.split(':');
        }
        const a1 = `${username}:${this.data.realm}:${password}`;
        const a2 = `${method}:${path}`;
        const response = `${md5(a1)}:${this.data.nonce}:${nc}:${cnonce}:${this.data.qop}:${md5(a2)}`;
        const digest = `Digest username="${username}", realm="${this.data.realm}", nonce="${this.data.nonce}", uri="${path}", response="${md5(response)}", qop=${this.data.qop}, nc=${nc}, cnonce="${cnonce}"`;
        return digest;
    }
    /**
     * NC取得
     */
    getNc() {
        return `${++this.nc}`.padStart(8, '0');
    }
}
exports.Challenge = Challenge;
function parseChallenge(challenge) {
    const data = {};
    const matches = challenge.match(/^digest (.+)$/i);
    if (matches) {
        matches[1].split(',').forEach((item) => {
            item = item.trim();
            let key, value;
            [key, value] = item.split('=');
            data[key] = JSON.parse(value);
        });
    }
    return data;
}
exports.parseChallenge = parseChallenge;
//# sourceMappingURL=challenge.js.map