"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const events_1 = require("events");
const logger_1 = require("./logger");
var TCPStreamEvent;
(function (TCPStreamEvent) {
    TCPStreamEvent["ReceiveData"] = "receive_data";
    TCPStreamEvent["SendData"] = "send_data";
    TCPStreamEvent["Close"] = "close";
    TCPStreamEvent["Error"] = "error";
})(TCPStreamEvent = exports.TCPStreamEvent || (exports.TCPStreamEvent = {}));
class TCPStream extends events_1.EventEmitter {
    constructor(tunnelName, socket, streamId, tcpClientHost, tcpClientPort) {
        super();
        this._tunnelName = tunnelName;
        this._streamId = streamId;
        this._socket = socket;
        this._receiveSize = 0;
        this._sendSize = 0;
        this._connectedTime = new Date().getTime();
        this._tcpClientHost = tcpClientHost;
        this._tcpClientPort = tcpClientPort;
        this._closedRemote = false;
        this._socket.on('data', (data) => {
            super.emit(TCPStreamEvent.ReceiveData, data);
            this._receiveSize += data.length;
        });
        this._socket.on('error', (e) => {
            this.throwError(e);
        });
        this._socket.on('close', () => {
            super.emit(TCPStreamEvent.Close, this.id);
        });
    }
    sendData(data) {
        super.emit(TCPStreamEvent.SendData, data);
        this._socket.write(data);
        this._sendSize += data.length;
    }
    close(closeFromRemoteTunnel = false) {
        this._closedRemote = closeFromRemoteTunnel;
        this._socket.end();
        this._socket.destroy();
    }
    throwError(error) {
        logger_1.Logger.systemError(error);
        super.emit(TCPStreamEvent.Error, error);
    }
    get tunnelName() {
        return this._tunnelName;
    }
    get id() {
        return this._streamId;
    }
    get socket() {
        return this._socket;
    }
    get receiveSize() {
        return this._receiveSize;
    }
    get sendSize() {
        return this._sendSize;
    }
    get readableHighWaterMark() {
        return this._socket.readableHighWaterMark;
    }
    get writableHighWaterMark() {
        return this._socket.writableHighWaterMark;
    }
    get tcpClientHost() {
        return this._tcpClientHost;
    }
    get tcpClientPort() {
        return this._tcpClientPort;
    }
    get connectedTime() {
        return this._connectedTime;
    }
    get closedRemote() {
        return this._closedRemote;
    }
}
exports.TCPStream = TCPStream;
//# sourceMappingURL=TCPStream.js.map