/// <reference types="node" />
import { EventEmitter } from 'events';
import { TunnelType, TunnelStatus, TunnelSetting } from './types';
import { TCPStream } from './TCPStream';
import * as net from 'net';
export declare type TunnelList = {
    [id: string]: Tunnel;
};
export declare type StreamList = {
    [id: string]: TCPStream;
};
export declare enum TunnelEvent {
    OpenStream = "open",
    CloseStream = "close",
    Listen = "listen",
    ReceiveDataFromTCP = "receive_data_from_tcp",
    Auth = "auth",
    CloseTunnel = "close_tunnel",
    Error = "error",
    ErrorLocalTCPServer = "error_local_tcp_server"
}
export declare class Tunnel extends EventEmitter {
    protected registerStream(socket: net.Socket, streamId: string, tcpClientHost: string, tcpClientPort: number): TCPStream;
    protected startLocalTCPServer(setting: TunnelSetting): void;
    openStream(streamId: string, tcpClientHost: string, tcpClientPort: number): TCPStream | undefined;
    sendDataToTcp(streamId: string, data: any): void;
    protected deleteStream(streamId: string): void;
    closeStream(streamId: string, fromRemote: boolean): void;
    close(): void;
    throwError(error: string): void;
    constructor(tunnelType: TunnelType, tunnelName: string, setting: TunnelSetting, approveOnNoSetAuthHandler?: boolean);
    get id(): string;
    get tunnelType(): TunnelType;
    get name(): string;
    get authId(): string;
    get authKey(): string;
    get status(): TunnelStatus;
    get tcpHost(): string;
    get tcpPort(): number;
    get listenPort(): number | undefined;
    set listenPort(val: number | undefined);
    get receiveSize(): number;
    get sendSize(): number;
    get streamCount(): number;
    private readonly _tunnelType;
    private readonly _tunnelId;
    private readonly _tunnelName;
    private readonly _tcpHost;
    private readonly _tcpPort;
    private _listenPort;
    private readonly _authId;
    private readonly _authKey;
    private _status;
    private _localTcpServer;
    private _localTcpServerInjected;
    private streams;
    private _receiveSize;
    private _sendSize;
}
