"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const pad8 = (num) => {
    let s = num + "";
    while (s.length < 8)
        s = "0" + s;
    return s;
};
const isIP = (addr) => {
    const regex = /^\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}(\/\d{1,2})?$/g;
    return addr.match(regex) != null;
};
const parseIpRange = (addrs) => {
    let [addr, mask] = addrs.split('/');
    const addrParts = addr.split('.');
    const netMask = parseInt(mask, 10) || 32;
    const addrBin = addrParts.map((p) => pad8(parseInt(p, 10).toString(2))).join('');
    return addrBin.substring(0, netMask);
};
exports.isAddrInSubNet = (addr, subnetAddr) => {
    if (isIP(addr) && isIP(subnetAddr)) {
        const addrBin = parseIpRange(addr);
        const subnetAddrBin = parseIpRange(subnetAddr);
        return addrBin.substring(0, subnetAddrBin.length) === subnetAddrBin;
    }
    else if (!isIP(addr) && !isIP(subnetAddr)) {
        return addr === subnetAddr;
    }
    return false;
};
exports.isPortInRange = (port, fromPort, toPort) => {
    if (port == 0)
        return true;
    return port >= fromPort && port <= toPort;
};
//# sourceMappingURL=util.js.map