import 'bootstrap/dist/css/bootstrap.min.css';
import 'izitoast/dist/css/izitoast.min.css';
import './css/simple-sidebar.css';
import './css/base.css';
import 'bootstrap';
import axios from 'axios';
import * as url from 'url';
import Vue from 'vue';
import VueI18n from 'vue-i18n';
import { socket } from './js/common';

import { boots } from '../../src/@types/boots';
import { status } from '../../src/@types/status';

// axios設定
axios.defaults.headers.common['X-Requested-With'] = 'XMLHttpRequest';
axios.defaults.headers.common['X-CSRF-TOKEN'] = (<HTMLMetaElement>document.head.querySelector('meta[name="csrf-token"]')).content;

// 多言語対応
Vue.use(VueI18n);
const messages = <VueI18n.LocaleMessages>{
    en: <VueI18n.LocaleMessage>require('../../resources/lang/en.json'),
    ja: <VueI18n.LocaleMessage>require('../../resources/lang/ja.json')
};
const i18n = new VueI18n({
    locale: document.documentElement.lang,
    messages
});

// コンポーネント
Vue.component('account', require('./js/components/account.vue').default);
Vue.component('connect', require('./js/components/connect.vue').default);
Vue.component('initPassword', require('./js/components/initPassword.vue').default);
Vue.component('lang', require('./js/components/lang.vue').default);
Vue.component('login', require('./js/components/login.vue').default);
Vue.component('password', require('./js/components/password.vue').default);
Vue.component('profile', require('./js/components/profile.vue').default);
Vue.component('proxy', require('./js/components/proxy.vue').default);
Vue.component('recovery', require('./js/components/recovery.vue').default);
Vue.component('registered', require('./js/components/registered.vue').default);
Vue.component('request', require('./js/components/request.vue').default);
Vue.component('status', require('./js/components/status.vue').default);

// 共通データ
Vue.mixin({
    data: function() {
        return {
            toast: window.toast,
            loading: window.loading,
            externalLink: window.externalLink
        }
    }
});

// フラッシュ通知
if(window['flash'].success) {
    window.toast.success(window['flash'].success);
}
if(window['flash'].error) {
    window.toast.error({message: window['flash'].error});
}

const app = new Vue({
    el: '#app',
    i18n,

    data() {
        return {
            loaded: false,
            socket: socket,
            status: <status>{
                code: null,
                message: null,
                registered: null,
                versionInfo: {
                    latestVersion: null,
                    latestUrl: null,
                    announcement: null
                }
            },
            boots: <boots>{
                browser: {
                    current: null,
                    max: null
                }
            }
        }
    },

    methods: {
        updateStatus: function(status: status) {
            this.status = status;
        },
        updateBoots: function(boots: boots) {
            this.boots.browser = boots.browser;
        },
        isSelected: function(name: string) {
            const parsed = url.parse(window.location.href)
            return {
                selected: parsed.path.indexOf(`/${name}`) === 0
            }
        }
    },

    mounted: function() {
        setTimeout(() => {
            this.loaded = true;
        });
    }
});
