import axios from 'axios';
import * as ioClient from 'socket.io-client';
import * as url from 'url';

const {host} = url.parse(document.location.href);
export const socket = ioClient.connect(host, {
    timeout: 10000,
    reconnectionAttempts: 6,
    transportOptions: {
        polling: {
            extraHeaders: {
                'X-CSRF-TOKEN': document.querySelector('meta[name="csrf-token"]').getAttribute('content')
            }
        }
    }
});

window.onload = function() {
    $("#menu-toggle").on('click', (e) => {
        e.preventDefault();
        $("#wrapper").toggleClass("toggled");
    });
}

const toast = require('izitoast');

declare global {
    interface Window {
        toast: {
            default: Function,
            success: Function,
            error: Function,
            confirm: Function
        },
        loading: {
            queue: string[],
            start: Function,
            end: Function
        },
        externalLink: Function
    }
}

window.toast = {
    default: (message: string, options={}) => {
        toast.show(Object.assign({
            class: 'toast-default',
            message: message,
            position: 'topCenter',
            drag: false,
            timeout: 5000,
            pauseOnHover: true,
            zindex: 999999,
            displayMode: 'replace',
        }, options));
    },
    success: (message: string, options={}) => {
        toast.success(Object.assign({
            class: 'toast-success',
            message: message,
            position: 'topCenter',
            drag: false,
            timeout: 5000,
            pauseOnHover: true,
            zindex: 999999,
            displayMode: 'replace',
        }, options));
    },
    error: (e: {message: string, errors?: string[]}, options={}) => {
        let message = e.message;
        if(e.errors) {
            message = `<ul class="mb-0">${e.errors.map((error) => {
                return `<li>${error}</li>`;
            }).join('')}</ul>`;
        }
        toast.error(Object.assign({
            class: 'toast-error',
            message: message,
            position: 'topCenter',
            drag: false,
            timeout: 5000,
            pauseOnHover: true,
            zindex: 999999,
            displayMode: 'replace',
        }, options));
    },
    confirm: (message: string, params) => {
        return <Promise<void>>new Promise((resolve,reject) => {
            if(document.querySelector('.toast')){
                toast.hide({},document.querySelector('.toast'));
            }
            const btn = {
                accept: params?.btn?.accept ? params.btn.accept : 'OK',
                dismiss: params?.btn?.dismiss ? params.btn.dismiss : 'Cancel',
            };
            toast.show({
                class: 'toast-confirm',
                close: false,
                message: message,
                position: 'topCenter',
                drag: false,
                timeout: 0,
                pauseOnHover: true,
                zindex: 999999,
                overlay: true,
                buttons: [
                    [`<button><b>${btn.accept}</b></button>`,(instance,toast) => {
                        instance.hide({transitionOut: 'fadeOut'},toast,'button');
                        resolve();
                    },true],
                    [`<button>${btn.dismiss}</button>`,(instance,toast) => {
                        instance.hide({transitionOut: 'fadeOut'},toast,'button');
                        reject();
                    }],
                ],
            });
        });
    }
}

window.loading = {
    queue: [],
    start: () => {
        return <Promise<void>>new Promise((resolve,reject) => {
            if(window.loading.queue.length){
                reject({message: '[loading]:reject'});
            }
            else{
                $('*').css('cursor','wait');
                const toastName = 'loading-' + (new Date().getTime());
                window.loading.queue.push(toastName);
                toast.show({
                    class: toastName,
                    message: '<i class="fas fa-spinner fa-spin"></i> Loading...',
                    color: 'yellow',
                    drag: false,
                    close: false,
                    overlay: false,
                    timeout: 0,
                    zindex: 999999,
                    onOpening: () => {
                        resolve();
                    }
                });
            }
        });
    },

    end: (toastName) => {
        if(!toastName){
            $('*').css('cursor','');
            toastName = window.loading.queue.shift();
            if(!toastName){
                return;
            }
        }
        if(!document.querySelector(`.${toastName}.iziToast-opening`)){
            toast.hide({}, document.querySelector('.' + toastName));
        }
        else{
            setTimeout(() => {
                window.loading.end(toastName);
            },10);
        }
    }
};

window.externalLink = async (url: string) => {
    event.preventDefault();
    const response = await axios.post('/api/external_link', {
        url: url
    });
    if(response.data.url) {
        window.open(response.data.url);
    }
}
