import axios from 'axios';
import { ComponentOptions } from 'vue';

export default <ComponentOptions<any>>{
    props: [
        'serviceName',
        'serviceUrl',
        'status'
    ],

    data: function() {
        return {
            agentContracts: null,
            agentRegistrationRequests: null,
            expired: null
        }
    },

    methods: {
        fetch: async function(): Promise<void> {
            try {
                this.agentContracts = null;
                this.agentRegistrationRequests = null;
                this.expired = null;

                const response = await axios.get('/api/agent/info');

                this.agentContracts = this.convertFormatAgentContracts(response.data.agent_contracts);
                this.agentRegistrationRequests = response.data.agent_registration_requests;
                this.expired = response.data.expired;
            }
            catch(e) {
                console.log(e.message);
                if(e.response?.data?.code == 401) {
                    e = new Error(`${this.$t('エージェントの認証に失敗しました。')}<br>${this.$t('新しいエージェントとして再接続してください。')}`);
                }
                else {
                    e = new Error(`${this.$t('接続エラーが発生しました。')}<br>${this.$t('インターネット接続およびプロキシの設定を確認してください。')}`);
                }
                this.toast.error(e);
                return;
            }
        },

        updateAccounts: async function(): Promise<void> {
            try {
                this.agentContracts = null;
                this.agentRegistrationRequests = null;
                this.expired = null;

                const response = await axios.get('/api/agent/accounts');

                this.agentContracts = this.convertFormatAgentContracts(response.data.agent_contracts);
                this.agentRegistrationRequests = response.data.agent_registration_requests;
                this.expired = response.data.expired;
            }
            catch(e) {
                console.log(e.message);
                if(e.response?.data?.code == 401) {
                    e = new Error(`${this.$t('エージェントの認証に失敗しました。')}<br>${this.$t('新しいエージェントとして再接続してください。')}`);
                }
                else {
                    e = new Error(`${this.$t('接続エラーが発生しました。')}<br>${this.$t('インターネット接続およびプロキシの設定を確認してください。')}`);
                }
                this.toast.error(e);
                return;
            }
        },

        convertFormatAgentContracts: function(agentContracts) {
            const list = {};
            agentContracts.forEach((agentContract) => {
                if(!list[agentContract.user]) {
                    list[agentContract.user] = {};
                }
                list[agentContract.user][agentContract.id] = agentContract.name;
            });
            return list;
        }
    },

    mounted: function() {
        setTimeout(async () => {
            await this.fetch();
        });
    }
}
