import axios from 'axios';
import { ComponentOptions } from 'vue';

export default <ComponentOptions<any>>{
    props: [
        'status'
    ],

    data: function() {
        return {
            email: null
        };
    },

    methods: {
        connect: async function(): Promise<void> {
            try {
                await this.loading.start();
            }
            catch(e) {
                console.log(e);
                return;
            }
            try {
                await axios.post('/api/agent/connect');
            }
            catch(e) {
                e = e.response?.data?.code ? e.response.data : e;
                this.toast.error(e);
                return;
            }
            finally {
                this.loading.end();
            }
        },
        disconnect: async function(): Promise<void> {
            try {
                await this.toast.confirm(this.$t('切断します。宜しいですか？'));
            }
            catch(e) {
                return;
            }

            try {
                await this.loading.start();
            } catch(e) {
                console.log(e);
                return;
            }
            try {
                await axios.delete('/api/agent/connect');
            }
            catch(e) {
                e = e.response?.data?.code ? e.response.data : e;
                this.toast.error(e);
                return;
            }
            finally {
                this.loading.end();
            }
        },
        resetAndConnect: async function(): Promise<void> {
            try {
                await this.loading.start();
            } catch(e) {
                console.log(e);
                return;
            }
            try {
                await axios.post('/api/agent/reset');
                axios.post('/api/agent/connect');
                document.location.reload();
            }
            catch(e) {
                e = e.response?.data?.code ? e.response.data : e;
                this.toast.error(e);
                return;
            }
            finally {
                this.loading.end();
            }
        }
    }
}
