import axios from 'axios';
import { ComponentOptions } from 'vue';

import { profile } from '../../../../src/@types/profile';

export default <ComponentOptions<any>>{
    props: [
        'feature',
        'profile',
        'wizard'
    ],

    data: function() {
        return <profile>{
            name: this.profile.name,
            max_values: {
                1: this.profile.max_values[1],
                2: this.profile.max_values[2]
            },
            allow_file_transfer: {
                inbound: this.profile.allow_file_transfer.inbound,
                outbound: this.profile.allow_file_transfer.outbound
            }
        }
    },

    methods: {
        update: async function() {
            if(!this.wizard) {
                try {
                    await this.toast.confirm(this.$t('基本設定を更新します。宜しいですか？'));
                }
                catch(e) {
                    return;
                }
            }

            try {
                await this.loading.start();
            }
            catch(e) {
                console.log(e);
                return;
            }
            try {
                await axios.post('/api/agent/profile', <profile>{
                    name: this.name,
                    max_values: this.max_values,
                    allow_file_transfer: this.allow_file_transfer
                });

                if(this.wizard) {
                    axios.post('/api/agent/connect');
                    document.location.href = '/';
                }
                else {
                    this.toast.success(this.$t('基本設定を更新しました。'));
                }
            }
            catch(e) {
                if(e.message == 'Network Error') {
                    e = new Error(`${this.$t('接続エラーが発生しました。')}<br>${this.$t('インターネット接続およびプロキシの設定を確認してください。')}`);
                }
                e = e.response?.data?.code ? e.response.data : e;
                this.toast.error(e);
                return;
            }
            finally {
                this.loading.end();
            }
        }
    }
}
