import axios from 'axios';
import { ComponentOptions } from 'vue';

export default <ComponentOptions<any>>{
    props: [
        'proxy',
        'status'
    ],

    data: function() {
        let url = null;
        try {
            url = new URL(this.proxy.url);
        }
        catch(e) {}

        if(this.proxy.readonly) {
            this.proxy.enable = false;
        }

        return {
            enable: this.proxy.enable,
            method: this.proxy.pac ? 'auto' : 'manual',
            host: url ? decodeURIComponent(url.hostname) : '',
            port: url ? url.port : '',
            userName: url ? decodeURIComponent(url.username) : '',
            password: url ? decodeURIComponent(url.password) : '',
            pac: this.proxy.pac
        };
    },

    methods: {
        update: async function() {
            try {
                await this.toast.confirm(`${this.$t('プロキシ設定を更新します。宜しいですか？')}<br><span class="small">${this.$t('オンラインの場合は一度切断して再接続します。')}</span>`);
            }
            catch(e) {
                return;
            }

            try {
                await this.loading.start();
            }
            catch(e) {
                console.log(e);
                return;
            }
            try {
                const payload = {
                    enable: this.enable
                };
                if(this.method == 'manual') {
                    Object.assign(payload, {
                        host: this.host,
                        port: Number(this.port),
                        userName: this.userName,
                        password: this.password
                    });
                }
                else {
                    Object.assign(payload, {
                        pac: this.pac
                    });
                }
                await axios.post('/api/agent/proxy', payload);
                this.toast.success(this.$t('プロキシ設定を更新しました。'));
            }
            catch(e) {
                e = e.response?.data?.code ? e.response.data : e;
                this.toast.error(e);
                return;
            }
            finally {
                this.loading.end();
            }
        }
    }
}
