import axios from 'axios';
import * as moment from 'moment';
import { ComponentOptions } from 'vue';

export default <ComponentOptions<any>>{
    props: [
        'agentContracts',
        'expired'
    ],

    methods: {
        release: async function(id): Promise<void> {
            try {
                await this.toast.confirm(this.$t('登録を解除します。宜しいですか？'));
            }
            catch(e) {
                return;
            }

            try {
                await this.loading.start();
            }
            catch(e) {
                console.log(e);
                return;
            }
            try {
                await axios.delete('/api/agent/registers', {
                    data: {
                        id: id
                    }
                });
                this.toast.success(this.$t('登録を解除しました。'));
                this.$emit('updateAccounts');
            }
            catch(e) {
                e = e.response?.data?.code ? e.response.data : e;
                this.toast.error(e);
                return;
            }
            finally {
                this.loading.end();
            }
        },
        countKeys: function(obj): number {
            let count = 0;
            for(const key in obj) {
                count++;
            }
            return count;
        }
    },

    filters: {
        moment: function(datetime: string) {
            return moment(datetime).format('YYYY-MM-DD HH:mm:ss');
        }
    }
}
