import axios from 'axios';
import * as moment from 'moment';
import { ComponentOptions } from 'vue';

export default <ComponentOptions<any>>{
    props: [
        'serviceName',
        'serviceUrl',
        'agentRegistrationRequests'
    ],

    data: function() {
        return {
            email: null
        };
    },

    methods: {
        requestRegistration: async function(): Promise<void> {
            try {
                await this.toast.confirm(this.$t('リクエストを送信します。宜しいですか？'));
            }
            catch(e) {
                return;
            }

            try {
                await this.loading.start();
            }
            catch(e) {
                console.log(e);
                return;
            }
            try {
                await axios.post('/api/agent/registration_requests', {
                    user: this.email
                });
                this.email = null;
                this.toast.success(this.$t('リクエストを送信しました。'));
                this.$emit('updateAccounts');
            }
            catch(e) {
                e = e.response?.data?.code ? e.response.data : e;
                this.toast.error(e);
                return;
            }
            finally {
                this.loading.end();
            }
        },
        cancel: async function(user) {
            try {
                await this.toast.confirm(this.$t('リクエストをキャンセルします。宜しいですか？'));
            }
            catch(e) {
                return;
            }

            try {
                await this.loading.start();
            }
            catch(e) {
                console.log(e);
                return;
            }
            try {
                await axios.delete('/api/agent/registration_requests', {
                    data: {
                        user: user
                    }
                });
                this.toast.success(this.$t('リクエストをキャンセルしました。'));
                this.$emit('updateAccounts');
            }
            catch(e) {
                e = e.response?.data?.code ? e.response.data : e;
                this.toast.error(e);
                return;
            }
            finally {
                this.loading.end();
            }
        }
    },

    computed: {
        serviceInfo: function() {
            return this.$t('{serviceName}アカウントをお持ちでない場合は、<a onclick="externalLink(\'{serviceUrl}\')" href="#"><u>こちら</u></a>からアカウント作成が可能です。', {serviceName: this.serviceName, serviceUrl: `${this.serviceUrl}/${document.documentElement.lang}`});
        }
    },

    filters: {
        moment: function(datetime: string) {
            return moment(datetime).format('YYYY-MM-DD HH:mm:ss');
        }
    }
}
