import { ComponentOptions } from 'vue';

import { boots } from '../../../../src/@types/boots';
import { status } from '../../../../src/@types/status';

export default <ComponentOptions<any>>{
    props: [
        'feature',
        'socket'
    ],

    data: function() {
        return {
            status: <status>{
                code: null,
                message: null,
                registered: null,
                versionInfo: {
                    latestVersion: null,
                    latestUrl: null,
                    announcement: null
                }
            },
            boots: <boots>{
                browser: {
                    current: null,
                    max: null
                },
                connector: {
                    current: null,
                    max: null
                }
            }
        }
    },

    mounted: function() {
        this.socket.on('disconnect', (): void => {
            this.status = <status>{
                code: 500,
                message: this.$t('エージェント終了'),
                registered: null,
                versionInfo: {
                    latestVersion: null,
                    latestUrl: null,
                    announcement: null
                }
            }
        });
        this.socket.on('connect_error', (): void => {
            this.status = <status>{
                code: 500,
                message: this.$t('エージェント終了'),
                registered: null,
                versionInfo: {
                    latestVersion: null,
                    latestUrl: null,
                    announcement: null
                }
            }
        });
        this.socket.on('statusNotification', (status: status): void => {
            this.status = status;
            this.$emit('update-status', this.status);
        });
        this.socket.on('bootsNotification', (boots: boots): void => {
            this.boots.browser = boots.browser;
            this.boots.connector = boots.connector;
            this.$emit('update-boots', this.boots);
        });
    },

    methods: {
        icon: function(): {[className: string]: boolean} {
            return {
                'fas fa-pause-circle': this.status.code == 0,
                'fas fa-circle-notch fa-spin': this.status.code >= 100 && this.status.code < 200,
                'fas fa-check-circle': this.status.code >= 200 && this.status.code < 300,
                'fas fa-exclamation-circle': this.status.code != 400 && this.status.code != 429 && this.status.code >= 400 && this.status.code < 500,
                'fas fa-exclamation-triangle': this.status.code == 400 || this.status.code == 429 || (this.status.code >= 500 && this.status.code < 600)
            }
        },

        color: function(): {[className: string]: boolean} {
            return {
                'alert-secondary': this.status.code == 0,
                'alert-info': this.status.code >= 100 && this.status.code < 200,
                'alert-success': this.status.code >= 200 && this.status.code < 300,
                'alert-warning': this.status.code != 400 && this.status.code != 429 && this.status.code >= 400 && this.status.code < 500,
                'alert-danger': this.status.code == 400 || this.status.code == 429 || (this.status.code >= 500 && this.status.code < 600)
            }
        }
    }
}
