"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const config = require("config");
const fs = require("fs");
const utils_1 = require("../utils");
const commandError_1 = require("../errors/commandError");
class default_1 {
    constructor(env) {
        this.env = env;
        this.agent_endpoint = config.agent.endpoint;
        this.agent_port = Number(this.env.agent_config.port.httpServer.split('-')[1]);
        try {
            this.readAgent();
        }
        catch (error) {
            // エージェント起動前は、ファイル存在しないことがあるためエラーを無視する
        }
    }
    /**
     * エージェント情報の読み込み
     */
    readAgent() {
        try {
            const data = fs.readFileSync(`${this.env.agent_config.apiRelayClient.agent.fileName}`, {
                encoding: this.env.agent_config.apiRelayClient.agent.encoding
            });
            const agent = JSON.parse(data.toString());
            this.agent_id = agent.id;
            this.agent_secret = agent.secret;
        }
        catch (error) {
            console.log(error.message);
            throw new commandError_1.CommandError('FAILED_READ_AGENT_FILE');
        }
    }
    /**
     * エージェントステータスの設定
     */
    setStatus(set_code) {
        try {
            if (fs.existsSync(`${this.env.agent_config.apiRelayClient.status.fileName}`)) {
                utils_1.writeFile(`${this.env.agent_config.apiRelayClient.status.fileName}`, JSON.stringify({
                    code: set_code
                }), this.env.agent_config.apiRelayClient.status.encoding);
            }
        }
        catch (error) {
            console.log(error.message);
            throw new commandError_1.CommandError('FAILED_UPDATE_AGENT_STATUS');
        }
    }
    /**
     * エージェントファイルが存在するかどうか
     */
    isExistAgnetFile() {
        return fs.existsSync(`${this.env.agent_config.apiRelayClient.agent.fileName}`);
    }
    /**
     * エージェントファイルが存在するかどうか
     */
    isExistProfileFile() {
        return fs.existsSync(`${this.env.agent_config.profile.fileName}`);
    }
}
exports.default = default_1;
