#!/usr/bin/env node
"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const commander_1 = require("commander");
const agent_1 = require("./agent");
const apiClient_1 = require("./apiClient");
const operation_1 = require("./operation");
const commandError_1 = require("../errors/commandError");
const environment_1 = require("../environment");
const app_1 = require("../app");
const manager_1 = require("../aliveMonitor/manager");
(() => __awaiter(void 0, void 0, void 0, function* () {
    var _a;
    try {
        commander_1.program
            .version((new app_1.default()).version, '-v, --version')
            .description('Disconnect the agent.')
            .option('--env-name <env name>', 'Add the agent environment variable name.')
            .parse(process.argv);
        // 引数
        const options = commander_1.program.opts();
        const env_name = (_a = options.envName) !== null && _a !== void 0 ? _a : 'default';
        // 環境情報
        const env = new environment_1.default(env_name);
        // エージェント情報
        const agent = new agent_1.default(env);
        // APIクライアント
        const apiClient = new apiClient_1.default(agent);
        // エージェント操作
        const agentOperation = new operation_1.default(agent, apiClient);
        // クラウドBOT用エージェントの場合、死活監視プロセスを終了する
        if (env.env_name == 'cbot') {
            const aliveMonitor = new manager_1.default(env);
            yield aliveMonitor.kill();
        }
        // エージェント終了
        yield agentOperation.exitProcess();
        console.log('Successful disconnection to the agent!!!');
        process.exit(0);
    }
    catch (error) {
        console.log('Failed to disconnect to the agent!!!');
        if (error instanceof commandError_1.CommandError) {
            console.log(`error: ${error.message}`);
            console.error(error.getJsonError());
            process.exit(error.getExitcode());
        }
        else {
            console.log('error: undefined error');
            console.error(JSON.stringify({
                exitcode: 999,
                message: error.message
            }));
            process.exit(999);
        }
    }
}))();
