"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const config = require("config");
const child_process_1 = require("child_process");
const commandError_1 = require("../errors/commandError");
const utils_1 = require("../utils");
class default_1 {
    constructor(agent, apiClient) {
        this.agent = agent;
        this.apiClient = apiClient;
    }
    /**
     * エージェントプロセス起動
     */
    bootProcess() {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                console.log('Boot agent...');
                const command = config.commands.agentBoot
                    .replace('{user}', config.agent.owner.user)
                    .replace('{env_name}', this.agent.env.env_name);
                const agentBootCommand = child_process_1.spawn(command, [], {
                    detached: true,
                    stdio: 'ignore',
                    shell: true
                });
                agentBootCommand.unref();
                console.log('done');
                return agentBootCommand.pid;
            }
            catch (error) {
                console.log(error.message);
                throw new commandError_1.CommandError('FAILED_BOOT_AGENT');
            }
        });
    }
    /**
     * エージェントプロセス終了
     */
    exitProcess() {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.agent.isExistAgnetFile()) {
                throw new commandError_1.CommandError('NOT_EXIST_AGENT_FILE');
            }
            console.log('Exit the agent process...');
            yield this.apiClient.exitProcess();
            console.log('done');
        });
    }
    /**
     * エージェントプロセスチェック
     */
    checkProcess() {
        return __awaiter(this, void 0, void 0, function* () {
            console.log('Check agent ready...');
            try {
                yield new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                    let ended = false;
                    const _resolve = () => {
                        ended = true;
                        clearTimeout(timer);
                        resolve();
                    };
                    const _reject = () => {
                        ended = true;
                        clearTimeout(timer);
                        reject();
                    };
                    // 10秒でタイムアウト
                    const timer = setTimeout(() => {
                        _reject();
                    }, 10000);
                    // 1秒に一回状態確認
                    while (!ended) {
                        yield utils_1.sleep(1000);
                        if (yield this.apiClient.isRunningAgent()) {
                            _resolve();
                        }
                    }
                }));
                // エージェント情報の読み込み
                this.agent.readAgent();
            }
            catch (error) {
                throw new commandError_1.CommandError('NO_AGENT_AVAILABLE');
            }
            ;
            console.log('done');
        });
    }
    /**
     * エージェントステータスをオフラインに変更
     */
    setAgentStatusOffline() {
        console.log('Initialize agent status...');
        this.agent.setStatus(0);
        console.log('done');
    }
    /**
     * エージェント初期化
     */
    initialize() {
        return __awaiter(this, void 0, void 0, function* () {
            console.log('Initialize agent...');
            yield this.apiClient.initialize();
            // エージェント情報の再読み込み
            this.agent.readAgent();
            console.log('done');
        });
    }
    /**
     * エージェント言語初期化
     */
    initLang(lang) {
        return __awaiter(this, void 0, void 0, function* () {
            console.log('Set language...');
            yield this.apiClient.initLang(lang);
            console.log('done');
        });
    }
    /**
     * エージェントプロフィール更新
     */
    updateProfile(agent_name, max_boots, allow_file_transfer, temp = false) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.agent.isExistProfileFile()) {
                console.log('Set agent profile...');
                yield this.apiClient.updateProfile(agent_name, max_boots, allow_file_transfer, temp);
                console.log('done');
            }
        });
    }
    /**
     * プロキシ設定の更新
     */
    reloadProxy() {
        return __awaiter(this, void 0, void 0, function* () {
            console.log('Set agent proxy...');
            yield this.apiClient.reloadProxy();
            console.log('done');
        });
    }
    /**
     * エージェント接続
     */
    connect() {
        return __awaiter(this, void 0, void 0, function* () {
            console.log('Agent connect...');
            yield this.apiClient.connect();
            // エージェント情報の再読み込み
            this.agent.readAgent();
            console.log('done');
        });
    }
    /**
     * エージェントPINリクエスト
     */
    inputPin(pin) {
        return __awaiter(this, void 0, void 0, function* () {
            console.log('Agent pin reception...');
            yield this.apiClient.inputPin(pin);
            console.log('done');
        });
    }
}
exports.default = default_1;
