#!/usr/bin/env node
"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const commander_1 = require("commander");
const config = require("config");
const commandError_1 = require("../errors/commandError");
const environment_1 = require("../environment");
const envNameRequest_1 = require("../requests/envNameRequest");
const utils_1 = require("../utils");
const child_process_1 = require("child_process");
const app_1 = require("../app");
(() => __awaiter(void 0, void 0, void 0, function* () {
    var _a;
    try {
        process.on('SIGINT', () => {
            console.log('End agent monitoring.');
            process.exit(0);
        });
        commander_1.program
            .version((new app_1.default()).version, '-v, --version')
            .description('Boot alive monitor.')
            .option('--env-name <env name>', 'Add the agent environment variable name.')
            .parse(process.argv);
        // 引数
        const options = commander_1.program.opts();
        const env_name = (_a = options.envName) !== null && _a !== void 0 ? _a : 'default';
        // 環境変数名のバリデーション
        const envNameRequest = new envNameRequest_1.default(env_name);
        envNameRequest.validate();
        // 環境情報
        const env = new environment_1.default(env_name);
        env.setAliveMonitorPid(process.pid);
        // １秒に一度エージェントの死活監視をする
        while (true) {
            // 1秒待機する
            yield utils_1.sleep(1000);
            console.log('Check agent process...');
            // エージェントプロセスが見つからない場合は、エージェントを起動する
            if (!env.existAgentPid()) {
                console.log('No agent process.');
                console.log('Start the agent...');
                const command = config.commands.startCbotAgent
                    .replace('{user}', config.agent.owner.user);
                const agentBootCommand = child_process_1.spawn(command, [], {
                    detached: true,
                    stdio: 'ignore',
                    shell: true
                });
                agentBootCommand.unref();
                console.log('done');
                process.exit(0);
            }
            console.log('Agent process found.');
        }
    }
    catch (error) {
        console.log('Alive monitor failed!!!');
        if (error instanceof commandError_1.CommandError) {
            console.log(`error: ${error.message}`);
            console.error(error.getJsonError());
            process.exit(error.getExitcode());
        }
        else {
            console.log('error: undefined error');
            console.error(JSON.stringify({
                exitcode: 999,
                message: error.message
            }));
            process.exit(999);
        }
    }
}))();
