"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const config = require("config");
const child_process_1 = require("child_process");
const commandError_1 = require("../errors/commandError");
class default_1 {
    constructor(env) {
        this.env = env;
    }
    /**
     * 起動
     */
    boot() {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                console.log('Boot alive monitor...');
                const command = config.commands.aliveMonitorBoot
                    .replace('{user}', config.agent.owner.user)
                    .replace('{env_name}', this.env.env_name);
                const aliveMonitorBootCommand = child_process_1.spawn(command, [], {
                    detached: true,
                    stdio: 'ignore',
                    shell: true
                });
                aliveMonitorBootCommand.unref();
                console.log('done');
            }
            catch (error) {
                console.log(error.message);
                throw new commandError_1.CommandError('FAILED_BOOT_ALIVE_MONITOR');
            }
        });
    }
    /**
     * 終了
     */
    kill() {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                if (this.env.pid.alive_monitor) {
                    console.log('Kill alive monitor...');
                    // PIDが死活監視プロセスではない場合は何もしない
                    try {
                        const get_process_name_command = config.commands.getProcessNameByPid
                            .replace('{pid}', this.env.pid.alive_monitor);
                        const process_name = child_process_1.execSync(get_process_name_command);
                        if (process_name.toString().indexOf('aliveMonitor/index.js') == -1) {
                            console.log('Not found alive monitor.');
                            return;
                        }
                    }
                    catch (error) {
                        // pidが存在しない場合は例外を無視する
                        console.log('Not found alive monitor.');
                        return;
                    }
                    // 子プロセスを全て取得する
                    let pid = this.env.pid.alive_monitor;
                    const pids = [];
                    while (pid) {
                        pids.push(pid);
                        try {
                            const get_child_process_pid_command = config.commands.getChildProcessPidByPpid
                                .replace('{ppid}', pid);
                            pid = parseInt(child_process_1.execSync(get_child_process_pid_command).toString());
                        }
                        catch (error) {
                            // 子プロセスが取れない場合はpidにnullを代入する
                            pid = null;
                        }
                    }
                    // 子プロセスから順にkillするためにpidを降順に並べ替える
                    pids.sort((a, b) => { return b - a; });
                    // プロセスにSIGINTシグナルを送る
                    pids.forEach(pid => {
                        const kill_process_command = config.commands.killProcessByPid
                            .replace('{pid}', pid);
                        child_process_1.execSync(kill_process_command);
                    });
                    console.log('done');
                }
            }
            catch (error) {
                console.log(error.message);
                throw new commandError_1.CommandError('FAILED_KILL_ALIVE_MONITOR');
            }
        });
    }
}
exports.default = default_1;
