"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const config = require("config");
const fs = require("fs");
const utils_1 = require("./utils");
const child_process_1 = require("child_process");
const commandError_1 = require("./errors/commandError");
class default_1 {
    constructor(env) {
        // モデル名
        this.model = '';
        // MACアドレス
        this.mac = '';
        this.env = env;
        try {
            this.readDeviceInfo();
        }
        catch (error) {
            // デバイス情報がない場合もあるため例外を無視する
        }
    }
    /**
     * デバイス情報の設定
     */
    readDeviceInfo() {
        const device_stdout = child_process_1.execSync(config.commands.getDeviceInfo);
        const deviceInfo = JSON.parse(device_stdout.toString());
        this.model = deviceInfo.model;
        this.mac = deviceInfo.mac;
    }
    /**
     * ハードウェアの設定
     */
    setHardware() {
        try {
            const hardware = {
                platform: null,
                platform_version: null,
                serial_number: null,
                detail: null
            };
            try {
                const data = fs.readFileSync(`${this.env.agent_config.hardware.fileName}`, {
                    encoding: this.env.agent_config.hardware.encoding
                });
                Object.assign(hardware, JSON.parse(data.toString()));
            }
            catch (e) {
                // ハードウェアファイルが存在しない場合もあるため例外を無視する
            }
            // 識別情報の設定
            hardware.serial_number = 'MAC Address: ' + this.mac;
            utils_1.writeFile(`${this.env.agent_config.hardware.fileName}`, JSON.stringify(hardware), this.env.agent_config.hardware.encoding);
        }
        catch (error) {
            console.log(error.message);
            throw new commandError_1.CommandError('FAILED_WRITE_HARDWARE_FILE');
        }
    }
}
exports.default = default_1;
