"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const config = require("config");
const parser = require("fast-xml-parser");
const fs = require("fs");
const commandError_1 = require("../errors/commandError");
class default_1 {
    constructor() {
        this.parseProxyXml();
    }
    /**
     * プロキシ設定XMLファイルの解析
     */
    parseProxyXml() {
        try {
            const data = fs.readFileSync(`${config.proxy.configXml}`);
            const proxy_info = parser.parse(data.toString(), {
                attributeNamePrefix: "",
                ignoreAttributes: false
            });
            this.proxy_enabled = this.toBoolean(proxy_info.proxy.enable);
            this.manual_setting = this.toBoolean(proxy_info.proxy.manual.enable);
            this.auto_setting = this.toBoolean(proxy_info.proxy.pacfile.enable);
            this.host = proxy_info.proxy.manual.host;
            this.port = Number(proxy_info.proxy.manual.port);
            if (proxy_info.proxy.manual.auth) {
                this.is_auth = this.toBoolean(proxy_info.proxy.manual.auth.enable);
                this.user = proxy_info.proxy.manual.auth.user;
                this.password = proxy_info.proxy.manual.auth.password;
            }
            else {
                this.is_auth = false;
                this.user = "";
                this.password = "";
            }
            this.pacfile_url = proxy_info.proxy.pacfile.url;
        }
        catch (error) {
            console.log(error.message);
            throw new commandError_1.CommandError('FAILED_PARSE_PROXY_CONFIG');
        }
    }
    /**
     * boolean変換
     */
    toBoolean(data) {
        if (!data) {
            return false;
        }
        return data.toLowerCase() === 'true';
    }
    /**
     * プロキシ情報のJSONオブジェクト
     */
    getProxyInfoByJson() {
        let url = "";
        let pac = "";
        if (this.proxy_enabled && this.auto_setting) {
            pac = this.pacfile_url;
        }
        else if (this.proxy_enabled && this.manual_setting) {
            let auth = "";
            if (this.is_auth) {
                auth = this.password ? `${this.user}:${this.password}@` : `${this.user}@`;
            }
            url = `http://${auth}${this.host}:${this.port}`;
        }
        return {
            "enable": this.proxy_enabled,
            "url": url,
            "pac": pac
        };
    }
}
exports.default = default_1;
