"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.mkdirAndChownRecursive = exports.writeFile = exports.sleep = void 0;
const config = require("config");
const fs = require("fs");
const path = require("path");
const child_process_1 = require("child_process");
/**
 * スリープ
 */
const sleep = (timeout) => {
    return new Promise((resolve, reject) => {
        setTimeout(() => {
            resolve();
        }, timeout);
    });
};
exports.sleep = sleep;
/**
 * ファイル書き込み
 */
const writeFile = (filepath, data, encode) => {
    // ディレクトリを再帰的に作成する
    exports.mkdirAndChownRecursive(filepath);
    // ファイルを作成し、オーナーを変更する
    fs.writeFileSync(filepath, data, {
        encoding: encode
    });
    changeOwner(filepath);
};
exports.writeFile = writeFile;
/**
 * 再帰的なディレクトリ作成とオーナー変更
 */
const mkdirAndChownRecursive = (filepath) => {
    let dir_name = '';
    const dir_names = path.dirname(filepath).split('/');
    dir_names.forEach(element => {
        if (element == '') {
            return;
        }
        dir_name += `/${element}`;
        // 対象ディレクトリが存在しない場合は、そのディレクトリを作成し、オーナーを変更する
        if (!fs.existsSync(dir_name)) {
            fs.mkdirSync(dir_name);
            changeOwner(dir_name);
        }
    });
};
exports.mkdirAndChownRecursive = mkdirAndChownRecursive;
/**
 * オーナーを変更する
 *
 * @param dir_name
 */
function changeOwner(dir_name) {
    // 実行環境がWindowsの場合は、処理しない
    if (process.platform != 'win32') {
        child_process_1.execSync(`chown ${config.agent.owner.user}:${config.agent.owner.group} ${dir_name}`);
    }
}
