# -*- coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta
from __future__ import print_function
from __future__ import unicode_literals
from __future__ import absolute_import
from __future__ import division

import os

from subprocess import Popen, PIPE
from product_conf import SUDO_CMD


SERVICE_NAME = 'cbot-agent'


def get_enabled():
    return get_service_enabled(SERVICE_NAME)

def set_enabled(enabled):
    success = set_service_enabled(SERVICE_NAME, enabled)
    if not enabled: _clear()  # TODO
    return success

def start():
    return start_service(SERVICE_NAME)

def stop():
    return stop_service(SERVICE_NAME)

# TODO
def _clear(dirname='/mnt/data/opt/cbot_agent/cbot', raise_exception=False):
    success = False
    try:
        from subprocess import Popen, PIPE
        from product_conf import SUDO_CMD as C_SUDO, RM_CMD as C_RM
        args = [C_SUDO, C_RM, '-r', dirname]
        proc = Popen(args, stdout=PIPE, stderr=PIPE)
        stdout, stderr = proc.communicate()
        success = proc.returncode == 0
        if raise_exception and not success:
            raise Exception(stderr)
    except Exception as e:
        if raise_exception:
            raise
    return success


# TODO
D_CURRENT = os.path.dirname(os.path.abspath(__file__))
F_AGREEMENTS = os.path.join(D_CURRENT, '_agreements')
def get_agreed(raise_exception=False):
    agreed = False
    try:
        args = [SUDO_CMD, F_AGREEMENTS, 'agreed']
        proc = Popen(args, stdout=PIPE, stderr=PIPE)
        stdout, stderr = proc.communicate()
        success = proc.returncode == 0
        if raise_exception and not success:
            raise Exception(stderr)
        yesno = stdout.strip()
        agreed = yesno == 'yes'
    except Exception as err:
        if raise_exception:
            raise
    return agreed
def agree(raise_exception=False):
    success = False
    try:
        args = [SUDO_CMD, F_AGREEMENTS, 'agree']
        proc = Popen(args, stdout=PIPE, stderr=PIPE)
        stdout, stderr = proc.communicate()
        success = proc.returncode == 0
        if raise_exception and not success:
            raise Exception(stderr)
    except Exception as err:
        if raise_exception:
            raise
    return success


##############################################################################


from fal_info import Results
from fal_info import FAL_SERVICECTRL_PATH
# from ionas.request import Request, Entry, EntryList, XmlConfNode
from ionas.request import Entry

from ._utils import send_entry, gettext, logging


def get_service_enabled(service_name, raise_exception=False, error_as=False):
    try:
        entry = Entry('query', '<servicectrl><servicelist><service><name/></service></servicelist></servicectrl>')
        servicecontrol_node = entry.get()
        services_node = servicecontrol_node.get_node('servicelist')
        service_node = services_node.get_node('service')
        name_node = service_node.get_node('name')
        name_node.set_text(service_name)
        _entry = send_entry(FAL_SERVICECTRL_PATH, entry)
        _result = _entry.get_result()
        _success = _result in (Results.OK,)
        if not _success:
            raise Exception('FALError')  # TODO
        _servicecontrol_node = _entry.get()
        _services_node = _servicecontrol_node.get_node('servicelist')
        _service_node = _services_node.get_node('service')
        _enabled = _service_node.is_enabled()
        return _enabled
    except Exception as e:
        if raise_exception:
            raise
        return error_as


def set_service_enabled(service_name, enabled, raise_exception=False):
    try:
        entry = Entry('apply', '<servicectrl><servicelist><service><name/></service></servicelist></servicectrl>')
        servicecontrol_node = entry.get()
        services_node = servicecontrol_node.get_node('servicelist')
        service_node = services_node.get_node('service')
        name_node = service_node.get_node('name')
        name_node.set_text(service_name)
        service_node.enable(enabled)
        _entry = send_entry(FAL_SERVICECTRL_PATH, entry)
        _result = _entry.get_result()
        _success = _result in (Results.OK,)
        if not _success:
            raise Exception('FALError')
        return _success
    except Exception as e:
        if raise_exception:
            raise
        return False


def start_service(service_name, raise_exception=False):
    try:
        entry = Entry('start', '<servicectrl><servicelist><service><name/></service></servicelist></servicectrl>')
        servicecontrol_node = entry.get()
        services_node = servicecontrol_node.get_node('servicelist')
        service_node = services_node.get_node('service')
        name_node = service_node.get_node('name')
        name_node.set_text(service_name)
        _entry = send_entry(FAL_SERVICECTRL_PATH, entry)
        _result = _entry.get_result()
        _success = _result in (Results.OK,)
        if not _success:
            raise Exception('FALError')
        return _success
    except Exception as e:
        if raise_exception:
            raise
        return False


def stop_service(service_name, raise_exception=False):
    try:
        entry = Entry('stop', '<servicectrl><servicelist><service><name/></service></servicelist></servicectrl>')
        servicecontrol_node = entry.get()
        services_node = servicecontrol_node.get_node('servicelist')
        service_node = services_node.get_node('service')
        name_node = service_node.get_node('name')
        name_node.set_text(service_name)
        _entry = send_entry(FAL_SERVICECTRL_PATH, entry)
        _result = _entry.get_result()
        _success = _result in (Results.OK,)
        if not _success:
            raise Exception('FALError')
        return _success
    except Exception as e:
        if raise_exception:
            raise
        return False
