#!/usr/bin/env python
# -*- coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta
from __future__ import print_function
from __future__ import unicode_literals
from __future__ import absolute_import
from __future__ import division

import os

from django import forms

from project.view import FuncView as _View

from . import fields
from .fal.service import (
    get_agreed,
    agree,
    get_enabled,
    set_enabled,
    start,
    stop,
    gettext,
    logging,
)

class CloudBOTAgentForm(forms.Form):

    enabled = fields.CloudBOTAgentEnabledField(label=u'Cloud BOT Agent')
    agreed = fields.CloudBOTAgentAgreedField(label=u'上記の内容に同意する')


class View(_View):

    views = ['apply']

    def view(self, request):
        form = CloudBOTAgentForm(initial=self._get_initial())

        already_agreed = get_agreed()

        template = os.path.join(self.TEMPLATES, 'template.html')
        dictionary = {
            'form': form,
            'already_agreed': already_agreed,
            'enabled': get_enabled(),
        }
        return {
            self.KEY_TEMPLATE: template,
            self.KEY_TEMPLATE_DIC: dictionary,
        }

    def view_apply(self, request):
        form = CloudBOTAgentForm(data=request.POST)

        already_agreed = get_agreed()
        with self.processing() as proc:
            if form.is_valid():
                enabled = form.cleaned_data['enabled']
                agreed = form.cleaned_data.get('agreed', False)
                success = False
                if agreed:
                    agree()
                already_agreed = get_agreed()
                if enabled:
                    if already_agreed:
                        success = set_enabled(enabled)
                        start()
                else:
                    stop()
                    success = set_enabled(enabled)
                proc.result(success)
                if success:
                    logging('MSG_SERVICE_CHANGE')
                    self.set_message(gettext('MSG_SERVICE_CHANGE'))
                else:
                    logging('ERR_SERVICE_CHANGE')
                    self.set_message(gettext('ERR_SERVICE_CHANGE'))

        template = os.path.join(self.TEMPLATES, 'template.html')
        dictionary = {
            'form': form,
            'already_agreed': already_agreed,
            'enabled': get_enabled(),
        }
        return {
            self.KEY_TEMPLATE: template,
            self.KEY_TEMPLATE_DIC: dictionary,
        }

    def _get_initial(self):
        return {
            'enabled': get_enabled(),
        }
